#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtklocalgeometry.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:48:31 $
## Version:   $Revision: 1.5 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkLocalGeometry(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.Centerlines = None
        self.VoronoiDiagram = None
        self.PoleIds = None

        self.EdgeArrayName = 'EdgeArray'

        self.VoronoiGeodesicDistanceArrayName = 'VoronoiGeodesicDistance'

        self.VoronoiPoleCenterlineVectorsArrayName = 'VoronoiPoleCenterlineVectors'
        self.VoronoiCellIdsArrayName = 'VoronoiCellIds'
        self.VoronoiPCoordsArrayName = 'VoronoiPCoords'

        self.ComputePoleVectors = 0
        self.ComputeGeodesicDistance = 0
        self.ComputeNormalizedTangencyDeviation = 0
        self.ComputeEuclideanDistance = 0
        self.ComputeCenterlineVectors = 0
        self.ComputeCellIds = 0
        self.ComputePCoords = 0

        self.AdjustBoundaryValues = 0

        self.PoleVectorsArrayName = 'PoleVectors'
        self.GeodesicDistanceArrayName = 'GeodesicDistance'
        self.NormalizedTangencyDeviationArrayName = 'NTD'
        self.EuclideanDistanceArrayName = 'EuclideanDistance'
        self.CenterlineVectorsArrayName = 'CenterlineVectors'
        self.CellIdsArrayName = 'CellIdsArray'
        self.PCoordsArrayName = 'PCoordsArray'

        self.SetScriptName('vmtklocalgeometry')
        self.SetScriptDoc('compute local geometric properties of a surface')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','','vmtksurfacereader'],
            ['Centerlines','centerlines','vtkPolyData',1,'','','vmtksurfacereader'],
            ['VoronoiDiagram','voronoidiagram','vtkPolyData',1,'','','vmtksurfacereader'],
            ['PoleIds','poleids','vtkIdList',1],
            ['EdgeArrayName','edgearray','str',1],
            ['VoronoiGeodesicDistanceArrayName','voronoigeodesicdistancearray','str',1],
            ['VoronoiPoleCenterlineVectorsArrayName','voronoipolecenterlinevectorsarray','str',1],
            ['VoronoiCellIdsArrayName','voronoicellidsarray','str',1],
            ['VoronoiPCoordsArrayName','voronoipcoordsarray','str',1],
            ['ComputePoleVectors','computepolevectors','bool',1],
            ['ComputeGeodesicDistance','computegeodesicdistance','bool',1],
            ['ComputeNormalizedTangencyDeviation','computentd','bool',1],
            ['ComputeEuclideanDistance','computeeuclideandistance','bool',1],
            ['ComputeCenterlineVectors','computecenterlinevectors','bool',1],
            ['ComputeCellIds','computecellids','bool',1],
            ['ComputePCoords','computepcoords','bool',1],
            ['AdjustBoundaryValues','adjustboundaryvalues','bool',1],
            ['PoleVectorsArrayName','polevectorsarray','str',1],
            ['GeodesicDistanceArrayName','geodesicdistancearray','str',1],
            ['NormalizedTangencyDeviationArrayName','ntdarray','str',1],
            ['EuclideanDistanceArrayName','euclideandistancearray','str',1],
            ['CenterlineVectorsArrayName','centerlinevectorsarray','str',1],
            ['CellIdsArrayName','cellidsarray','str',1],
            ['PCoordsArrayName','pcoordsarray','str',1]
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','','vmtksurfacewriter'],
            ['PoleVectorsArrayName','polevectorsarray','str',1],
            ['GeodesicDistanceArrayName','geodesicdistancearray','str',1],
            ['NormalizedTangencyDeviationArrayName','ntdarray','str',1],
            ['EuclideanDistanceArrayName','euclideandistancearray','str',1],
            ['CenterlineVectorsArrayName','centerlinevectorsarray','str',1],
            ['CellIdsArrayName','cellidsarray','str',1],
            ['PCoordsArrayName','pcoordsarray','str',1]
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if self.Centerlines == None:
            self.PrintError('Error: No input centerlines.')

        if self.VoronoiDiagram == None:
            self.PrintError('Error: No input Voronoi diagram.')

        if self.PoleIds == None:
            self.PrintError('Error: No input pole ids.')

        voronoi = self.VoronoiDiagram

        if self.ComputeGeodesicDistance | self.ComputeNormalizedTangencyDeviation | self.ComputeEuclideanDistance | self.ComputeCenterlineVectors | self.ComputeCellIds | self.ComputePCoords:

            if self.EdgeArrayName == '':
                self.PrintError('Error: No input edge array name.')

            voronoiRadialFastMarching = vtkvmtk.vtkvmtkNonManifoldFastMarching()
            voronoiRadialFastMarching.SetInputData(voronoi)
            voronoiRadialFastMarching.UnitSpeedOn()
            voronoiRadialFastMarching.SetSolutionArrayName(self.VoronoiGeodesicDistanceArrayName)
            voronoiRadialFastMarching.PolyDataBoundaryConditionsOn()
            voronoiRadialFastMarching.SetBoundaryPolyData(self.Centerlines)
            voronoiRadialFastMarching.SetIntersectedEdgesArrayName(self.EdgeArrayName)
            voronoiRadialFastMarching.Update()
            voronoi = voronoiRadialFastMarching.GetOutput()

        if self.ComputeEuclideanDistance | self.ComputeCenterlineVectors | self.ComputeCellIds | self.ComputePCoords:

            voronoiShooter = vtkvmtk.vtkvmtkSteepestDescentShooter()
            voronoiShooter.SetInputData(voronoi)
            voronoiShooter.SetTarget(self.Centerlines)
            voronoiShooter.SetDescentArrayName(self.VoronoiGeodesicDistanceArrayName)
            voronoiShooter.SetEdgeArrayName(self.EdgeArrayName)
            voronoiShooter.SetSeeds(self.PoleIds)
            voronoiShooter.SetTargetVectorsArrayName(self.VoronoiPoleCenterlineVectorsArrayName)
            voronoiShooter.SetTargetCellIdsArrayName(self.VoronoiCellIdsArrayName)
            voronoiShooter.Update()
            voronoi = voronoiShooter.GetOutput()

        surfaceLocalGeometry = vtkvmtk.vtkvmtkPolyDataLocalGeometry()

        surfaceLocalGeometry.SetInputData(self.Surface)
        surfaceLocalGeometry.SetVoronoiDiagram(voronoi)
        surfaceLocalGeometry.SetVoronoiGeodesicDistanceArrayName(self.VoronoiGeodesicDistanceArrayName)
        surfaceLocalGeometry.SetPoleIds(self.PoleIds)

        surfaceLocalGeometry.SetVoronoiPoleCenterlineVectorsArrayName(self.VoronoiPoleCenterlineVectorsArrayName)
        surfaceLocalGeometry.SetVoronoiCellIdsArrayName(self.VoronoiCellIdsArrayName)
        surfaceLocalGeometry.SetVoronoiPCoordsArrayName(self.VoronoiPCoordsArrayName)

        surfaceLocalGeometry.SetComputePoleVectors(self.ComputePoleVectors)
        surfaceLocalGeometry.SetComputeGeodesicDistance(self.ComputeGeodesicDistance)
        surfaceLocalGeometry.SetComputeNormalizedTangencyDeviation(self.ComputeNormalizedTangencyDeviation)
        surfaceLocalGeometry.SetComputeEuclideanDistance(self.ComputeEuclideanDistance)
        surfaceLocalGeometry.SetComputeCenterlineVectors(self.ComputeCenterlineVectors)
        surfaceLocalGeometry.SetComputeCellIds(self.ComputeCellIds)
        surfaceLocalGeometry.SetComputePCoords(self.ComputePCoords)

        surfaceLocalGeometry.SetAdjustBoundaryValues(self.AdjustBoundaryValues)

        surfaceLocalGeometry.SetPoleVectorsArrayName(self.PoleVectorsArrayName)
        surfaceLocalGeometry.SetGeodesicDistanceArrayName(self.GeodesicDistanceArrayName)
        surfaceLocalGeometry.SetNormalizedTangencyDeviationArrayName(self.NormalizedTangencyDeviationArrayName)
        surfaceLocalGeometry.SetEuclideanDistanceArrayName(self.EuclideanDistanceArrayName)
        surfaceLocalGeometry.SetCenterlineVectorsArrayName(self.CenterlineVectorsArrayName)
        surfaceLocalGeometry.SetCellIdsArrayName(self.CellIdsArrayName)
        surfaceLocalGeometry.SetPCoordsArrayName(self.PCoordsArrayName)

        surfaceLocalGeometry.Update()

        self.Surface = surfaceLocalGeometry.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshextractpointdata.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.6 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Simone Manini
##       Orobix Srl

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
from vmtk import vtkvmtk
import vtk
import sys

from vmtk import pypes
from vmtk import vmtkmeshvectorfromcomponents


class vmtkMeshExtractPointData(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Mesh = None
        self.PointId = None
        self.ArrayName = None
        self.ComponentsArrayNames = None
        self.SetScriptName('vmtkmeshextractpointdata')
        self.SetScriptDoc('extract the magnitude value of an array for a point id')
        self.SetInputMembers([
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['PointId','pointid','int',1,'','the id of the cell'],
            ['ComponentsArrayNames','components','str',-1,'',''],
            ['ArrayName','array','str',1,'',''],
            ])
        self.SetOutputMembers([
            ])

    def Execute(self):

        if (self.Mesh == None):
            self.PrintError('Error: no Mesh.')

        if (self.PointId == None):
            self.PrintError('Error: no PointId.')

        if (self.ArrayName == None):
            self.PrintError('Error: no ArrayName.')

        vectorFromComponents = vmtkmeshvectorfromcomponents.vmtkMeshVectorFromComponents()
        vectorFromComponents.Mesh = self.Mesh
        vectorFromComponents.VectorArrayName = self.ArrayName
        vectorFromComponents.ComponentsArrayNames = self.ComponentsArrayNames
        vectorFromComponents.RemoveComponentArrays = True
        vectorFromComponents.Execute()
        dataArray = self.Mesh.GetPointData().GetArray(self.ArrayName)
        dataValue = vtk.vtkMath.Norm((dataArray.GetTuple(self.PointId)[0],dataArray.GetTuple(self.PointId)[1],dataArray.GetTuple(self.PointId)[2]))

        self.PrintLog('')

        logLine = 'PointId: %d\n' % self.PointId
        logLine += '  Array: %s\n' % self.ArrayName
        logLine += ' Components[x,y,z]: %s,%s,%s\n' % (dataArray.GetTuple(self.PointId)[0],dataArray.GetTuple(self.PointId)[1],dataArray.GetTuple(self.PointId)[2])
        logLine += '  Magnitude: %f\n' % dataValue

        self.PrintLog(logLine)


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkcenterlinegeometry as centerlinegeometry


def test_default_parameters(aorta_centerline, compare_centerlines):
    name = __name__ + '_test_default_parameters.vtp'
    geometry = centerlinegeometry.vmtkCenterlineGeometry()
    geometry.Centerlines = aorta_centerline
    geometry.Execute()

    assert compare_centerlines(geometry.Centerlines, name, method='addcellarray', arrayname='Tortuosity') == True
    assert compare_centerlines(geometry.Centerlines, name, method='addpointarray', arrayname='Torsion') == True
    assert compare_centerlines(geometry.Centerlines, name, method='addpointarray', arrayname='Curvature') == True
    assert compare_centerlines(geometry.Centerlines, name, method='addcellarray', arrayname='Length') == True
    assert compare_centerlines(geometry.Centerlines, name, method='addpointarray', arrayname='FrenetTangent') == True
    assert compare_centerlines(geometry.Centerlines, name, method='addpointarray', arrayname='FrenetNormal') == True
    assert compare_centerlines(geometry.Centerlines, name, method='addpointarray', arrayname='FrenetBinormal') == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkpeterresurfaceclipper.py,v $
## Language:  Python
## Date:      JAN 05, 2021
## Version:   1.4

##   Copyright (c) Peter Patalano, Luca Antiga. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Peter Patalano (Github @ppatalano
## * Added AutoClip functionality which allows clips from a mesh to be
##   stored then used again on a future mesh
## * Useful for meshes that have undergone a stenosis following CFD and
##   need to remove old extensions to place new ones


from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vmtkrenderer
from vmtk import pypes


class vmtkPeterSurfaceClipper(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.ClippedSurface = None
        self.CutLines = None
        self.vmtkRenderer = None
        self.OwnRenderer = 0

        self.WidgetType = 'box'

        self.Actor = None
        self.ClipWidget = None
        self.ClipFunction = None
        self.CleanOutput = 1
        self.Transform = None

        self.InsideOut = 0

        self.Interactive = 1
        self.ClipArrayName = None
        self.ClipValue = 0.0

        # Globals for our added functionality
        self.ClipsIn = None
        self.ClipsOut = None
        self.ClipCollection = None

        self.SetScriptName('petersurfaceclipper')
        self.SetScriptDoc('interactively clip a surface with a box')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            # The new input member which will be a MultiBlock file
            ['ClipsIn','iclip','vtkMultiBlockDataSet',1,'','the input clips file','vmtksurfacereader'],
            ['WidgetType','type','str',1,'["box","sphere"]','type of widget used for clipping'],
            ['Transform','transform','vtkTransform',1,'','the widget transform, useful in case of piping of multiple clipping scripts'],
            ['CleanOutput','cleanoutput','bool',1,'','toggle cleaning the unused points'],
            ['InsideOut','insideout','bool',1,'','toggle switching output and clipped surfaces'],
            ['Interactive','interactive','bool',1,'','toggle clipping with a widget or with a pre-defined scalar field'],
            ['ClipArrayName','array','str',1,'','name of the array with which to clip in case of non-interactive clipping'],
            ['ClipValue','value','float',1,'','scalar value at which to perform clipping in case of non-interactive clipping'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter'],
            ['ClippedSurface','oclipped','vtkPolyData',1,'','the clipped surface','vmtksurfacewriter'],
            ['CutLines','ocutlines','vtkPolyData',1,'','the cutlines','vmtksurfacewriter'],
            # The new otput member which will be a MultiBlock file
            ['ClipsOut','oclip','vtkMultiBlockDataSet',-1,'','the output surfaces associated with the clips made','vmtksurfacewriter'],
            ['Transform','otransform','vtkTransform',1,'','the output widget transform']
            ])

    def CleanAutoClip(self):
        if not self.Surface.IsA('vtkPolyData'):
            self.PrintError('Error: CleanAutoClip input is not PolyData.')
            return

        # Use connectivty to clean data of any fragments generated
        Filter = vtk.vtkPolyDataConnectivityFilter()
        Filter.SetInputData(self.Surface)
        Filter.Update()
        self.PrintLog('Distinct regions: ' + str(Filter.GetNumberOfExtractedRegions()))

        if Filter.GetNumberOfExtractedRegions == 1:
            self.PrintLog('Single region identified')
            return

        Filter.SetExtractionModeToLargestRegion()
        Filter.Update()
        self.Surface.DeepCopy(Filter.GetOutput())

    def AutoClip(self):
        # Check to see if ClipsIn is provided
        if self.ClipsIn == None:
            self.PrintError('Error: no AutoClip without ClipsIn')

        # Init AutoClipper/Function
        AutoClipper = vtk.vtkClipPolyData()
        AutoClipFunction = vtk.vtkPlanes()

        AutoClipper.SetInputData(self.Surface)
        AutoClipper.GenerateClippedOutputOn()
        AutoClipper.SetInsideOut(self.InsideOut)
        AutoClipper.SetClipFunction(AutoClipFunction)

        # Perform ClipFunction while ClipsIn is not empty
        while not self.ClipsIn.GetNumberOfBlocks() == 0:
            Bounds = self.NextBlockBounds()
            self.PrintLog('Bounds for AutoClip are: ' + str(Bounds))
            AutoClipFunction.SetBounds(Bounds)
            AutoClipper.Update()
            self.Surface.DeepCopy(AutoClipper.GetOutput())

        import pdb; pdb.set_trace()
        self.CleanAutoClip()
        self.PrintLog('AutoClip complete.')

    def NextBlockBounds(self):
        if self.ClipsIn == None:
            self.PrintError('Error: no clip input provided')

        # Get the index for the last block in the dataset
        LastBlock = self.ClipsIn.GetNumberOfBlocks() - 1

        # Get data for the last block
        BlockData = vtk.vtkPolyData()
        BlockData.DeepCopy(self.ClipsIn.GetBlock(LastBlock))
        self.ClipsIn.RemoveBlock(LastBlock)
        Bounds = BlockData.GetBounds()
        return Bounds

    def GetClipData(self):
        #Initialize ClipCollection
        if self.ClipCollection == None:
            self.ClipCollection = vtk.vtkPolyDataCollection()

        #Get PolyData of the current box, requires deep copy
        dummy = vtk.vtkPolyData()
        ClipPolyData = vtk.vtkPolyData()
        self.ClipWidget.GetPolyData(dummy)
        ClipPolyData.DeepCopy(dummy)

        #Add ClipPolyData to the ClipCollection
        self.ClipCollection.AddItem(ClipPolyData)

        #Display a running list of clips made
        self.ClipCollection.InitTraversal()
        self.PrintLog('All bounds are: ')
        for i in range(self.ClipCollection.GetNumberOfItems()):
            a = self.ClipCollection.GetNextItem()
            self.PrintLog(str(a.GetBounds()))

    def ClipCallback(self, obj):
        if self.ClipWidget.GetEnabled() != 1:
            return

        if self.WidgetType == "box":
            #self.ClipWidget.SetPlaceFactor(1)
            #self.ClipWidget.PlaceWidget(self.NextBlockBounds())
            self.Display()
            self.ClipWidget.GetPlanes(self.ClipFunction)

        elif self.WidgetType == "sphere":
            self.ClipWidget.GetSphere(self.ClipFunction)
        self.GetClipData()
        self.Clipper.Update()
        self.Surface.DeepCopy(self.Clipper.GetOutput())
        self.ClippedSurface.DeepCopy(self.Clipper.GetClippedOutput())
        self.Cutter.Update()
        self.CutLines.DeepCopy(self.Cutter.GetOutput())
        self.ClipWidget.Off()

    def InteractCallback(self, obj):
        pass
    #    if self.ClipWidget.GetEnabled() == 1:
    #        self.ClipWidget.SetEnabled(0)
    #    else:
    #        self.ClipWidget.SetEnabled(1)

    def Display(self):
        self.ClipWidget.SetInputData(self.Surface)
        self.ClipWidget.PlaceWidget()

        # If ClipsIn provided, display given bounds of last block
        #if self.ClipsIn and self.WidgetType == "box":
        #    self.ClipWidget.SetPlaceFactor(1)
        #    self.ClipWidget.PlaceWidget(self.NextBlockBounds())
        #    self.ClipWidget.On()

        if self.Transform and self.WidgetType == "box":
            self.PrintLog('ClipsIn is None')
            self.ClipWidget.SetTransform(self.Transform)
            self.ClipWidget.On()

        #self.vmtkRenderer.RenderWindowInteractor.Initialize()
        self.vmtkRenderer.Render()
        self.PrintLog('End of Display()')

        #self.vmtkRenderer.RenderWindowInteractor.Start()

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: no Surface to for Execute().')

        self.Clipper = vtk.vtkClipPolyData()
        self.Clipper.SetInputData(self.Surface)
        self.Clipper.GenerateClippedOutputOn()
        self.Clipper.SetInsideOut(self.InsideOut)

        if self.Interactive:

            if self.WidgetType == "box":
                self.ClipFunction = vtk.vtkPlanes()
            elif self.WidgetType == "sphere":
                self.ClipFunction = vtk.vtkSphere()

            self.Clipper.SetClipFunction(self.ClipFunction)

            self.Cutter = vtk.vtkCutter()
            self.Cutter.SetInputData(self.Surface)
            self.Cutter.SetCutFunction(self.ClipFunction)

            self.ClippedSurface = vtk.vtkPolyData()
            self.CutLines = vtk.vtkPolyData()

            if not self.vmtkRenderer:
                self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
                self.vmtkRenderer.Initialize()
                self.OwnRenderer = 1

            self.vmtkRenderer.RegisterScript(self)

            mapper = vtk.vtkPolyDataMapper()
            mapper.SetInputData(self.Surface)
            mapper.ScalarVisibilityOff()
            self.Actor = vtk.vtkActor()
            self.Actor.SetMapper(mapper)
            self.vmtkRenderer.Renderer.AddActor(self.Actor)
            if self.WidgetType == "box":
                self.ClipWidget = vtk.vtkBoxWidget()
                self.ClipWidget.GetFaceProperty().SetColor(0.6,0.6,0.2)
                self.ClipWidget.GetFaceProperty().SetOpacity(0.25)
            elif self.WidgetType == "sphere":
                self.ClipWidget = vtk.vtkSphereWidget()
                self.ClipWidget.GetSphereProperty().SetColor(0.6,0.6,0.2)
                self.ClipWidget.GetSphereProperty().SetOpacity(0.25)
                self.ClipWidget.GetSelectedSphereProperty().SetColor(0.6,0.0,0.0)
                self.ClipWidget.GetSelectedSphereProperty().SetOpacity(0.75)
                self.ClipWidget.SetRepresentationToSurface()
                self.ClipWidget.SetPhiResolution(20)
                self.ClipWidget.SetThetaResolution(20)

            self.ClipWidget.SetInteractor(self.vmtkRenderer.RenderWindowInteractor)
            # Try AutoClip here
            if not self.ClipsIn == None:
                self.AutoClip()

            self.vmtkRenderer.AddKeyBinding('space','Clip.',self.ClipCallback)
            self.vmtkRenderer.AddKeyBinding('i','Interact.',self.InteractCallback)
            self.Display()

            self.Transform = vtk.vtkTransform()
            if self.WidgetType == "box":
                self.ClipWidget.GetTransform(self.Transform)

            # Add items from ClipCollection to MultiBlockDataSet
            self.ClipsOut = vtk.vtkMultiBlockDataSet()
            ClipTotal = self.ClipCollection.GetNumberOfItems()
            BlockNum = 0
            self.ClipCollection.InitTraversal()
            while BlockNum < ClipTotal:
                blockin = vtk.vtkPolyData()
                blockin.DeepCopy(self.ClipCollection.GetNextItem())
                self.PrintLog('The bounds of blockin are: ' + str(blockin.GetBounds()))
                self.ClipsOut.SetBlock(BlockNum, blockin)
                BlockNum += 1

            if self.OwnRenderer:
                self.vmtkRenderer.Deallocate()

        else:

            self.Surface.GetPointData().SetActiveScalars(self.ClipArrayName)

            self.Clipper.GenerateClipScalarsOff()
            self.Clipper.SetValue(self.ClipValue)
            self.Clipper.Update()

            self.Cutter = vtk.vtkContourFilter()
            self.Cutter.SetInputData(self.Surface)
            self.Cutter.SetValue(0,self.ClipValue)
            self.Cutter.Update()

            self.Surface = self.Clipper.GetOutput()
            self.ClippedSurface = self.Clipper.GetClippedOutput()

            self.CutLines = self.Cutter.GetOutput()

        if self.CleanOutput == 1:

            cleaner = vtk.vtkCleanPolyData()
            cleaner.SetInputData(self.Surface)
            cleaner.Update()
            self.Surface = cleaner.GetOutput()

            cleaner = vtk.vtkCleanPolyData()
            cleaner.SetInputData(self.ClippedSurface)
            cleaner.Update()
            self.ClippedSurface = cleaner.GetOutput()

            cleaner = vtk.vtkCleanPolyData()
            cleaner.SetInputData(self.CutLines)
            cleaner.Update()
            stripper = vtk.vtkStripper()
            stripper.SetInputConnection(cleaner.GetOutputPort())
            stripper.Update()
            self.CutLines = stripper.GetOutput()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      February 7, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkbifurcationreferencesystems as bifurcationreferencesystems
from vtk.numpy_interface import dataset_adapter as dsa
import numpy as np


@pytest.fixture(scope='module')
def bifurcation_reference_system(aorta_centerline_branches):
    bifur = bifurcationreferencesystems.vmtkBifurcationReferenceSystems()
    bifur.Centerlines = aorta_centerline_branches
    bifur.RadiusArrayName = 'MaximumInscribedSphereRadius'
    bifur.BlankingArrayName = 'Blanking'
    bifur.GroupIdsArrayName = 'GroupIds'
    bifur.Execute()

    return bifur.ReferenceSystems


def test_reference_system_composed_of_3_arrays(bifurcation_reference_system):
    dataArrays = bifurcation_reference_system.GetPointData()
    assert dataArrays.GetNumberOfArrays() == 3


def test_array_0_name_is_normal(bifurcation_reference_system):
    dataArrays = bifurcation_reference_system.GetPointData()
    assert dataArrays.GetArrayName(0) == 'Normal'


def test_array_0_has_three_components(bifurcation_reference_system):
    dataArrays = bifurcation_reference_system.GetPointData()
    array = dataArrays.GetArray(0)
    assert array.GetNumberOfComponents() == 3


def test_array_1_name_is_upnormal(bifurcation_reference_system):
    dataArrays = bifurcation_reference_system.GetPointData()
    assert dataArrays.GetArrayName(1) == 'UpNormal'


def test_array_1_has_three_components(bifurcation_reference_system):
    dataArrays = bifurcation_reference_system.GetPointData()
    array = dataArrays.GetArray(1)
    assert array.GetNumberOfComponents() == 3


def test_array_2_name_is_groupids(bifurcation_reference_system):
    dataArrays = bifurcation_reference_system.GetPointData()
    assert dataArrays.GetArrayName(2) == 'GroupIds'


def test_array_2_has_one_components(bifurcation_reference_system):
    dataArrays = bifurcation_reference_system.GetPointData()
    array = dataArrays.GetArray(2)
    assert array.GetNumberOfComponents() == 1


def test_reference_system_contains_one_points(bifurcation_reference_system):
    assert bifurcation_reference_system.GetNumberOfPoints() == 1


def test_bifurcation_point_at_expected_location(bifurcation_reference_system):
    expectedLocation = np.array([221.66326904296875, 138.3250732421875, 24.043575286865234])
    assert np.allclose(np.array(bifurcation_reference_system.GetPoint(0)), expectedLocation) == True


def test_normal_array_values_match_expected(bifurcation_reference_system):
    dataArrays = bifurcation_reference_system.GetPointData()
    array = dataArrays.GetArray('Normal')
    computedValue = np.array([array.GetComponent(0, 0), array.GetComponent(0, 1), array.GetComponent(0, 2)])
    expectedValue = np.array([-0.16795282, -0.05915027, -0.98401885])

    assert np.allclose(computedValue, expectedValue) == True


def test_upnormal_array_values_match_expected(bifurcation_reference_system):
    dataArrays = bifurcation_reference_system.GetPointData()
    array = dataArrays.GetArray('UpNormal')
    computedValue = np.array([array.GetComponent(0, 0), array.GetComponent(0, 1), array.GetComponent(0, 2)])
    expectedValue = np.array([-0.00365111, -0.99815405,  0.06062313])

    assert np.allclose(computedValue, expectedValue) == True


def test_groupids_array_values_match_expected(bifurcation_reference_system):
    dataArrays = bifurcation_reference_system.GetPointData()
    array = dataArrays.GetArray('GroupIds')
    computedValue = np.array([array.GetComponent(0, 0)])
    expectedValue = np.array([1.0])

    assert np.allclose(computedValue, expectedValue) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkmeshvolume.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkMeshVolume(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Mesh = None

        self.Volume = 0.0

        self.SetScriptName('vmtkmeshvolume')
        self.SetScriptDoc('computes the volume of a mesh')
        self.SetInputMembers([
            ['Mesh','i','vtkPolyData',1,'','the input mesh','vmtkmeshreader']
            ])
        self.SetOutputMembers([
            ['Volume','volume','float',1,'']
            ])

    def Execute(self):

        if self.Mesh == None:
            self.PrintError('Error: No input mesh.')

        tetrahedralizeFilter = vtk.vtkDataSetTriangleFilter()
        tetrahedralizeFilter.SetInputData(self.Mesh)
        tetrahedralizeFilter.TetrahedraOnlyOn()
        tetrahedralizeFilter.Update()

        self.Mesh = tetrahedralizeFilter.GetOutput()

        self.Volume = 0.0
        cellPoints = vtk.vtkIdList()
        for i in range(self.Mesh.GetNumberOfCells()):
            self.Mesh.GetCellPoints(i,cellPoints)
            point0 = self.Mesh.GetPoint(cellPoints.GetId(0))
            point1 = self.Mesh.GetPoint(cellPoints.GetId(1))
            point2 = self.Mesh.GetPoint(cellPoints.GetId(2))
            point3 = self.Mesh.GetPoint(cellPoints.GetId(3))
            self.Volume += abs(vtk.vtkTetra.ComputeVolume(point0,point1,point2,point3))


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      February 7, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkcenterlineoffsetattributes as centerlineoffsetattributes
import vmtk.vmtksurfacereader as surfacereader
import os
from vtk.numpy_interface import dataset_adapter as dsa
import numpy as np


@pytest.fixture(scope='module')
def reference_system(input_datadir):
    reader = surfacereader.vmtkSurfaceReader()
    reader.InputFileName = os.path.join(input_datadir, 'aorta-centerline-referencesystem.vtp')
    reader.Execute()
    return reader.Surface


@pytest.fixture(scope='module')
def centerline_attribute_branched(input_datadir):
    reader = surfacereader.vmtkSurfaceReader()
    reader.InputFileName = os.path.join(input_datadir, 'aorta-centerline-attribute-branches.vtp')
    reader.Execute()
    return reader.Surface


def test_compare_offset_abscissa(reference_system, centerline_attribute_branched, compare_centerlines):
    name = __name__ + '_test_compare_offset_abscissa.vtp'
    offset = centerlineoffsetattributes.vmtkCenterlineOffsetAttributes()
    offset.Centerlines = centerline_attribute_branched
    offset.ReferenceSystems = reference_system
    offset.AbscissasArrayName = "Abscissas"
    offset.NormalsArrayName = "ParallelTransportNormals"
    offset.GroupIdsArrayName = "GroupIds"
    offset.CenterlineIdsArrayName = "CenterlineIds"
    offset.ReferenceSystemsNormalArrayName = "Normal"
    offset.OffsetAbscissasArrayName = "OffsetAbscissas"
    offset.OffsetNormalsArrayName = "OffsetNormals"
    offset.ReferenceGroupId = 1
    offset.Execute()

    assert compare_centerlines(offset.Centerlines, name, method='addpointarray', arrayname='Abscissas') == True


def test_compare_offset_abscissa_and_normal_without_inplace_modification(reference_system,
                                                                        centerline_attribute_branched,
                                                                        compare_centerlines):
    name = __name__ + '_test_compare_offset_abscissa_and_normal_without_inplace_modification.vtp'
    offset = centerlineoffsetattributes.vmtkCenterlineOffsetAttributes()
    offset.Centerlines = centerline_attribute_branched
    offset.ReferenceSystems = reference_system
    offset.AbscissasArrayName = "Abscissas"
    offset.NormalsArrayName = "ParallelTransportNormals"
    offset.GroupIdsArrayName = "GroupIds"
    offset.CenterlineIdsArrayName = "CenterlineIds"
    offset.ReferenceSystemsNormalArrayName = "Normal"
    offset.OffsetAbscissasArrayName = "OffsetAbscissas"
    offset.OffsetNormalsArrayName = "OffsetNormals"
    offset.ReferenceGroupId = 1
    offset.ReplaceAttributes = 0
    offset.Execute()

    assert compare_centerlines(offset.Centerlines, name, method='addpointarray', arrayname='OffsetAbscissas') == True
    assert compare_centerlines(offset.Centerlines, name, method='addpointarray', arrayname='OffsetNormals') == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimagereslice.py,v $
## Language:  Python
## Date:      $Date: 2006/07/17 09:53:14 $
## Version:   $Revision: 1.8 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
from vmtk import vtkvmtk
import vtk
import sys

from vmtk import pypes


class vmtkRBFInterpolation(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Seeds = None
        self.RBFType = 'biharmonic'

        self.Image = None

        self.Dimensions = [0, 0, 0]
        self.Bounds = [0.0, 1.0, 0.0, 1.0, 0.0, 1.0]

        self.SetScriptName('vmtkrbfinterpolation')
        self.SetScriptDoc('perform RBF interpolation from a set of seeds')
        self.SetInputMembers([
            ['Seeds','i','vtkPolyData',1,'','the input seeds','vmtksurfacereader'],
            ['Image','r','vtkImageData',1,'','the reference image','vmtkimagereader'],
            ['Dimensions','dimensions','int',3,''],
            ['Bounds','bounds','float',6,''],
            ['RBFType','rbftype','str',1,'["thinplatespline","biharmonic","triharmonic"]','the type of RBF interpolation']

            ])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter']
            ])

    def Execute(self):

        if self.Seeds == None:
            self.PrintError('Error: No input seeds.')

        rbf = vtkvmtk.vtkvmtkRBFInterpolation()
        rbf.SetSource(self.Seeds)
        if self.RBFType == "thinplatespline":
            rbf.SetRBFTypeToThinPlateSpline()
        elif self.RBFType == "biharmonic":
            rbf.SetRBFTypeToBiharmonic()
        elif self.RBFType == "triharmonic":
            rbf.SetRBFTypeToTriharmonic()
        rbf.ComputeCoefficients()

        if self.Image:
            origin = self.Image.GetOrigin()
            spacing = self.Image.GetSpacing()
            extent = self.Image.GetExtent()
            dimensions = self.Image.GetDimensions()
            modelBounds = [0.0,0.0,0.0,0.0,0.0,0.0]
            modelBounds[0] = origin[0] + spacing[0]*extent[0]
            modelBounds[1] = origin[0] + spacing[0]*extent[1]
            modelBounds[2] = origin[1] + spacing[1]*extent[2]
            modelBounds[3] = origin[1] + spacing[1]*extent[3]
            modelBounds[4] = origin[2] + spacing[2]*extent[4]
            modelBounds[5] = origin[2] + spacing[2]*extent[5]
        else:
            dimensions = self.Dimensions
            modelBounds = self.Bounds

        sampleFunction = vtk.vtkSampleFunction()
        sampleFunction.SetImplicitFunction(rbf)
        sampleFunction.SetOutputScalarTypeToDouble()
        sampleFunction.SetSampleDimensions(dimensions)
        sampleFunction.SetModelBounds(modelBounds)
        sampleFunction.CappingOff()
        sampleFunction.ComputeNormalsOff()
        sampleFunction.Update()

        self.Image = sampleFunction.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python


from __future__ import print_function, absolute_import # NEED TO STAY AS TOP IMPORT
import vtk
import sys
import math
from vmtk import vtkvmtk


def ReadPolyData(filename):
   reader = vtk.vtkXMLPolyDataReader()
   reader.SetFileName(filename)
   reader.Update()
   return reader.GetOutput()


def WritePolyData(input,filename):
   writer = vtk.vtkXMLPolyDataWriter()
   writer.SetInputData(input)
   writer.SetFileName(filename)
   writer.Write()


def MaskVoronoiDiagram(voronoi,centerlines):
   numberOfCenterlinesPatches = centerlines.GetNumberOfCells()
   numberOfVoronoiPoints = voronoi.GetNumberOfPoints()

   maskArray = vtk.vtkIntArray()
   maskArray.SetNumberOfComponents(1)
   maskArray.SetNumberOfTuples(numberOfVoronoiPoints)
   maskArray.FillComponent(0,0)

   for i in range(numberOfCenterlinesPatches):
     tangent,center,centerMISR = ComputePatchEndPointParameters(i,centerlines)
     MaskWithPatch(i,tangent,center,centerMISR,maskArray,centerlines,voronoi)
     maskedPoints = ComputeNumberOfMaskedPoints(maskArray)

   return maskArray


def ComputePatchEndPointParameters(id,centerlines):
   point0 = [0.0,0.0,0.0]
   point1 = [0.0,0.0,0.0]
   tan = [0.0,0.0,0.0]
   radius0 = -1

   cell = vtk.vtkGenericCell()
   centerlines.GetCell(id,cell)

   if (id == 0):
     point0 = cell.GetPoints().GetPoint(cell.GetNumberOfPoints()-1)
     point1 = cell.GetPoints().GetPoint(cell.GetNumberOfPoints()-2)
     radius0 = centerlines.GetPointData().GetArray(radiusArrayName).GetTuple1(cell.GetPointId(cell.GetNumberOfPoints()-1))

     tan[0] = point1[0]-point0[0]
     tan[1] = point1[1]-point0[1]
     tan[2] = point1[2]-point0[2]
     vtk.vtkMath.Normalize(tan)
   else:
     point0 = cell.GetPoints().GetPoint(0)
     point1 = cell.GetPoints().GetPoint(1)
     radius0 = centerlines.GetPointData().GetArray(radiusArrayName).GetTuple1(cell.GetPointId(0))

     tan[0] = point1[0]-point0[0]
     tan[1] = point1[1]-point0[1]
     tan[2] = point1[2]-point0[2]
     vtk.vtkMath.Normalize(tan)
   return tan, point0, radius0


def MaskWithPatch(id,t,c,r,maskArray,centerlines,voronoi):
   patch = ExtractPatch(id,centerlines)

   tubeFunction = vtkvmtk.vtkvmtkPolyBallLine()
   tubeFunction.SetInput(patch)
   tubeFunction.SetPolyBallRadiusArrayName(radiusArrayName)

   lastSphere = vtk.vtkSphere()
   lastSphere.SetRadius(r*1.5)
   lastSphere.SetCenter(c)

   for i in range(voronoi.GetNumberOfPoints()):
      point = [0.0,0.0,0.0]
      voronoiVector = [0.0,0.0,0.0]

      voronoi.GetPoint(i,point)
      voronoiVector[0] = point[0]-c[0]
      voronoiVector[1] = point[1]-c[1]
      voronoiVector[2] = point[2]-c[2]
      voronoiVectorDot = vtk.vtkMath.Dot(voronoiVector,t)

      tubevalue = tubeFunction.EvaluateFunction(point)
      spherevalue = lastSphere.EvaluateFunction(point)
      if (spherevalue<0.0) & (voronoiVectorDot<0.0): continue
      elif (tubevalue<=0.0):
         maskArray.SetTuple1(i,1)


def ExtractPatch(id,centerlines):
   cell = vtk.vtkGenericCell()
   centerlines.GetCell(id,cell)
   numberOfPoints = cell.GetNumberOfPoints()

   line = vtk.vtkPolyData()
   cellArray = vtk.vtkCellArray()
   cellArray.InsertNextCell(numberOfPoints)

   linePoints = cell.GetPoints()

   radiusArray = vtk.vtkDoubleArray()
   radiusArray.SetNumberOfComponents(1)
   radiusArray.SetNumberOfTuples(numberOfPoints)
   radiusArray.SetName(radiusArrayName)
   radiusArray.FillComponent(0,0.0)

   for i in range(numberOfPoints):
      cellArray.InsertCellPoint(i)
      radius = centerlines.GetPointData().GetArray(radiusArrayName).GetTuple1(cell.GetPointId(i))
      radiusArray.SetTuple1(i,radius)

   line.SetPoints(linePoints)
   line.SetLines(cellArray)
   line.GetPointData().AddArray(radiusArray)
   return line


def ComputeNumberOfMaskedPoints(dataArray):
   numberOfPoints = 0
   for i  in range(dataArray.GetNumberOfTuples()):
      value = dataArray.GetTuple1(i)
      if (value ==1): numberOfPoints +=1
   return numberOfPoints


def ExtractMaskedVoronoiPoints(voronoi,maskArray):
   numberOfPoints = ComputeNumberOfMaskedPoints(maskArray)

   maskedVoronoi = vtk.vtkPolyData()
   maskedPoints = vtk.vtkPoints()
   cellArray = vtk.vtkCellArray()

   radiusArray = vtk.vtkDoubleArray()
   radiusArray.SetNumberOfComponents(1)
   radiusArray.SetNumberOfTuples(numberOfPoints)
   radiusArray.SetName(radiusArrayName)
   radiusArray.FillComponent(0,0.0)

   count = 0
   for i in range(voronoi.GetNumberOfPoints()):
      point = [0.0,0.0,0.0]
      voronoi.GetPoint(i,point)
      pointRadius = voronoi.GetPointData().GetArray(radiusArrayName).GetTuple1(i)
      value = maskArray.GetTuple1(i)
      if (value ==1):
        maskedPoints.InsertNextPoint(point)
        radiusArray.SetTuple1(count,pointRadius)
        cellArray.InsertNextCell(1)
        cellArray.InsertCellPoint(count)
        count +=1

   maskedVoronoi.SetPoints(maskedPoints)
   maskedVoronoi.SetVerts(cellArray)
   maskedVoronoi.GetPointData().AddArray(radiusArray)
   return maskedVoronoi


def SmoothClippedVoronoiDiagram(voronoi,centerlines):
   numberOfPoints = voronoi.GetNumberOfPoints()
   numberOfCenterlinesPoints = centerlines.GetNumberOfPoints()

   maskArray = vtk.vtkIntArray()
   maskArray.SetNumberOfComponents(1)
   maskArray.SetNumberOfTuples(numberOfPoints)
   maskArray.FillComponent(0,0)

   for i in range(numberOfCenterlinesPoints):
     localRadius = centerlines.GetPointData().GetArray(radiusArrayName).GetTuple1(i)

     threshold = localRadius*(1.0 - smoothingFactor)

     sphere = vtk.vtkSphere()
     sphere.SetRadius(localRadius)
     sphere.SetCenter(centerlines.GetPoint(i))

     localMaskArray = vtk.vtkIntArray()
     localMaskArray.SetNumberOfComponents(1)
     localMaskArray.SetNumberOfTuples(numberOfPoints)
     localMaskArray.FillComponent(0,0)

     for j in range(numberOfPoints):
        value = sphere.EvaluateFunction(voronoi.GetPoint(j))
        if (value <= 0.0):
           localMaskArray.SetTuple1(j,1)

     for j in range(numberOfPoints):
       value = localMaskArray.GetTuple1(j)
       if (value==1):
          r = voronoi.GetPointData().GetArray(radiusArrayName).GetTuple1(j)
          if (r>threshold):
             maskArray.SetTuple1(j,1)

   finalNumberOfMaskedPoints = ComputeNumberOfMaskedPoints(maskArray)
   print('from original number of points ', numberOfPoints,'to',finalNumberOfMaskedPoints)

   smoothedDiagram = vtk.vtkPolyData()
   points = vtk.vtkPoints()
   cellArray = vtk.vtkCellArray()

   radiusArray = vtk.vtkDoubleArray()
   radiusArray.SetNumberOfComponents(1)
   radiusArray.SetNumberOfTuples(finalNumberOfMaskedPoints)
   radiusArray.FillComponent(0,0.0)
   radiusArray.SetName(radiusArrayName)

   count = 0
   for i in range(numberOfPoints):
      value = maskArray.GetTuple1(i)
      if (value==1):
         radius = voronoi.GetPointData().GetArray(radiusArrayName).GetTuple1(i)

         points.InsertNextPoint(voronoi.GetPoint(i))
         cellArray.InsertNextCell(1)
         cellArray.InsertCellPoint(count)
         radiusArray.SetTuple1(count,radius)
         count +=1

   smoothedDiagram.SetPoints(points)
   smoothedDiagram.SetVerts(cellArray)
   smoothedDiagram.GetPointData().AddArray(radiusArray)
   return smoothedDiagram


# -----------------------------------------------------------------------------------------


## Program:	clipvoronoidiagram.py
## Language:	Python
## Date:	2012/02/27
## Version:	1.0
## Application: Cerebral Aneurysms - Parent Vessel Reconstruction
## Author:	Marina Piccinelli

## Description:	Keeps the Voronoi Diagram points around to the "patched" centerlines and
##		removes all the rest. If smoothing option is selected (default and
##		recommended) provides a smoothed version of Voronoi Diagram; points from
##		the Voronoi Diagram are removed depending on the value of the maximum
##		inscribed sphere radius.

# -----------------------------------------------------------------------------------------


#SOME COMMON VMTK DATA ARRAY NAMES
radiusArrayName = 'MaximumInscribedSphereRadius'

#OPTIONS TO SET
smoothVoronoiDiagram = 1	# recommended; perform smoothing of the Voronoi Diagram to
##				remove smaller maximal spheres
smoothingFactor = 0.25 		# usually set to  0.25; the bigger the smoothing factor the
##				smaller the number of removed spheres.

#inputfilename:
print('USAGE:')
print('      ./clipvoronoidiagram.py inputfilesDirectory caseID')
print('')

inputfiledirectory = sys.argv[1]
ID = sys.argv[2]

print('Inputfiles Directory	', inputfiledirectory)
print('case ID			', ID)
print('')

voronoiDiagramFilename   = inputfiledirectory + '/' + ID + '/' + ID + '_voronoi.vtp'
patchCenterlinesFilename = inputfiledirectory + '/' + ID + '/' + ID + '_patchcl.vtp'

#outputfilenames:
clippedVoronoiDiagramFilename = inputfiledirectory + '/' + ID + '/' + ID + '_clippedvoronoi.vtp'
smoothVoronoiDiagramFilename  = inputfiledirectory + '/' + ID + '/' + ID + '_smoothclippedvoronoi.vtp'

originalVoronoiDiagram = ReadPolyData(voronoiDiagramFilename)
patchCenterlines       = ReadPolyData(patchCenterlinesFilename)

print('Clipping Voronoi Diagram')
maskedVoronoiArray    = MaskVoronoiDiagram(originalVoronoiDiagram,patchCenterlines)
clippedVoronoiDiagram = ExtractMaskedVoronoiPoints(originalVoronoiDiagram,maskedVoronoiArray)
WritePolyData(clippedVoronoiDiagram, clippedVoronoiDiagramFilename)

if (smoothVoronoiDiagram==1):
   print('Smoothing Voronoi Diagram')
   smoothVoronoiDiagram = SmoothClippedVoronoiDiagram(clippedVoronoiDiagram,patchCenterlines)
   WritePolyData(smoothVoronoiDiagram,smoothVoronoiDiagramFilename)



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtknumpytoimage.py,v $
## Language:  Python
## Date:      June 10, 2017
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo
##       The Jacobs Institute

from __future__ import absolute_import  # NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import pypes

from vtk.numpy_interface import dataset_adapter as dsa
from vtk.util import numpy_support

try:
    import numpy as np
except ImportError:
    raise ImportError('Unable to Import vmtkimagetonumpy module, numpy is not installed')


# convenience class for nested dictionaries
class vividict(dict):
    def __missing__(self, key):
        value = self[key] = type(self)()
        return value


class vmtkNumpyToImage(pypes.pypeScript):
    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.ArrayDict = None

        self.SetScriptName('vmtkNumpyToImage')
        self.SetScriptDoc('Takes a dictionary of numpy arrays specifying dimensions, origin, spacing, and the image '
                          'point data and returns a vtkImageData object.')
        self.SetInputMembers([
            ['ArrayDict','i','dict',1,'','the input array dictionary','vmtknumpyreader']])
        self.SetOutputMembers([
            ['Image','o','vtkImageData',1,'','the output image','vmtkimagewriter']])

    def Execute(self):

        self.PrintLog('Converting Numpy Array to vtkImageData')
        self.Image = vtk.vtkImageData()
        self.Image.SetDimensions(self.ArrayDict['Dimensions'])
        self.Image.SetOrigin(self.ArrayDict['Origin'])
        self.Image.SetSpacing(self.ArrayDict['Spacing'])
        self.Image.SetExtent((0, self.ArrayDict['Dimensions'][0] - 1,
                                0, self.ArrayDict['Dimensions'][1] - 1,
                                0, self.ArrayDict['Dimensions'][2] - 1,))

        self.PrintLog('converting point data')
        for pointKey in self.ArrayDict['PointData'].keys():
            if np.issubdtype(self.ArrayDict['PointData'][pointKey].dtype, np.floating):
                pointDataArrayType = vtk.VTK_FLOAT
            else:
                for checkDt in [np.integer, np.uint8, np.uint16, np.uint32, np.uint64]:
                    if np.issubdtype(self.ArrayDict['PointData'][pointKey].dtype, checkDt):
                        pointDataArrayType = vtk.VTK_INT
                        break
                    else:
                        continue

            flatArray = self.ArrayDict['PointData'][pointKey].ravel(order='F')

            pointDataArray = dsa.numpyTovtkDataArray(flatArray, name=pointKey, array_type=pointDataArrayType)

            self.Image.GetPointData().SetActiveScalars(pointKey)
            self.Image.GetPointData().SetScalars(pointDataArray)


if __name__ == '__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkimagemipviewer.py,v $
## Language:  Python
## Date:      $Date: 2006/06/20 12:12:05 $
## Version:   $Revision: 1.1 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import pypes


class vmtkImageMIPViewer(pypes.pypeScript):
    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Image = None
        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.Display = 1
        self.ArrayName = ''

        self.SampleDistance = 1.0
        self.AutoSampleDistance = 1

        self.WindowLevel = [0.0, 0.0]
        self.Opacity = None

        self.Volume = None

        self.SetScriptName('vmtkimagemipviewer')
        self.SetScriptDoc('display a 3D image')
        self.SetInputMembers([
            ['Image', 'i', 'vtkImageData', 1, '', 'the input image', 'vmtkimagereader'],
            ['ArrayName', 'array', 'str', 1, '', 'name of the array to display'],
            ['vmtkRenderer', 'renderer', 'vmtkRenderer', 1, '', 'external renderer'],
            ['SampleDistance', 'sampledistance', 'float', 1, '(0.0,)',
             'the distance at sample projections are generated'],
            ['AutoSampleDistance', 'autosampledistance', 'bool', 1, '', 'toggle automatic sample distance'],
            ['WindowLevel', 'windowlevel', 'float', 2, '', 'the window/level for generating the rendering'],
            ['Opacity', 'opacity', 'float', 2, '', 'the opacity range is defined from 0 (transparent) to 1 (opaque)'],
            ['Display', 'display', 'bool', 1, '', 'toggle rendering']
        ])
        self.SetOutputMembers([
            ['Image', 'o', 'vtkImageData', 1, '', 'the output image', 'vmtkimagewriter']
        ])

    def BuildView(self):

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self)

        if self.Volume:
            self.vmtkRenderer.Renderer.RemoveVolume(self.Volume)

        if (self.ArrayName != ''):
            self.Image.GetPointData().SetActiveScalars(self.ArrayName)

        scalarRange = [0.0, 0.0]

        if self.WindowLevel[0] > 0.0:
            scalarRange = [self.WindowLevel[1] - self.WindowLevel[0] / 2.0,
                           self.WindowLevel[1] + self.WindowLevel[0] / 2.0]
        else:
            scalarRange = self.Image.GetScalarRange()

        colorTransferFunction = vtk.vtkColorTransferFunction()
        colorTransferFunction.AddRGBPoint(scalarRange[0], 0.0, 0.0, 0.0)
        colorTransferFunction.AddRGBPoint(scalarRange[1], 1.0, 1.0, 1.0)

        volumeMapper = vtk.vtkFixedPointVolumeRayCastMapper()
        volumeMapper.SetInputData(self.Image)
        volumeMapper.SetBlendModeToMaximumIntensity()
        if self.AutoSampleDistance:
            volumeMapper.AutoAdjustSampleDistancesOn()
        else:
            volumeMapper.SetSampleDistance(self.SampleDistance)

        opacityFunction = vtk.vtkPiecewiseFunction()
        imageRange = self.Image.GetScalarRange()

        if self.Opacity is not None:
            opacityFunction.AddPoint(imageRange[0], 0)
            opacityFunction.AddPoint(scalarRange[0], self.Opacity[0])
            opacityFunction.AddPoint(scalarRange[0], self.Opacity[1])
            opacityFunction.AddPoint(imageRange[1], 1)

        volumeProperty = vtk.vtkVolumeProperty()
        volumeProperty.ShadeOn()
        volumeProperty.SetInterpolationTypeToLinear()
        volumeProperty.SetColor(colorTransferFunction)
        if self.Opacity is not None:
            volumeProperty.SetScalarOpacity(opacityFunction)

        self.Volume = vtk.vtkVolume()
        self.Volume.SetMapper(volumeMapper)
        self.Volume.SetProperty(volumeProperty)

        self.vmtkRenderer.Renderer.AddVolume(self.Volume)

        if (self.Display == 1):
            self.vmtkRenderer.Render()

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()

    def Execute(self):

        if (self.Image == None) & (self.Display == 1):
            self.PrintError('Error: no Image.')

        self.BuildView()


if __name__ == '__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtktetgen.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.7 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import sys
import vtk
from vmtk import vtkvmtk

from vmtk import pypes


class vmtkTetGen(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Mesh = None
        self.GenerateCaps = 0

        self.PLC = 1
        self.Refine = 0
        self.Coarsen = 0
        self.NoBoundarySplit = 0
        self.Quality = 1
        self.MinRatio = 1.414
        self.MinDihedral = 10.0
        self.MaxDihedral = 165.0
        self.VarVolume = 0
        self.FixedVolume = 0
        self.MaxVolume = 1E-1
        self.RemoveSliver = 0
        self.RegionAttrib = 0
        self.Epsilon = 1E-8
        self.NoMerge = 0
        self.DetectInter = 0
        self.CheckClosure = 0
        self.Order = 1
        self.DoCheck = 0
        self.Verbose = 0
        self.UseSizingFunction = 0
        self.SizingFunctionArrayName = 'VolumeSizingFunction'

        self.CellEntityIdsArrayName = 'CellEntityIds'
        self.TetrahedronVolumeArrayName = 'TetrahedronVolume'

        self.OutputSurfaceElements = 1
        self.OutputVolumeElements = 1

        self.SetScriptName('vmtktetgen')
        self.SetScriptDoc('wrapper around TetGen tetrahedral mesh generator by Hang Si (http://tetgen.berlios.de/)')
        self.SetInputMembers([
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['GenerateCaps','caps','bool',1,'','close surface holes with caps before meshing'],
            ['PLC','plc','int',1,'','see TetGen documentation'],
            ['Refine','refine','int',1,'','see TetGen documentation'],
            ['Coarsen','coarsen','int',1,'','see TetGen documentation'],
            ['NoBoundarySplit','noboundarysplit','int',1,'','see TetGen documentation'],
            ['Quality','quality','int',1,'','see TetGen documentation'],
            ['MinRatio','minratio','float',1,'','see TetGen documentation'],
            ['MinDihedral','mindihedral','float',1,'','see TetGen documentation'],
            ['MaxDihedral','maxdihedral','float',1,'','see TetGen documentation'],
            ['VarVolume','varvolume','int',1,'','see TetGen documentation'],
            ['FixedVolume','fixedvolume','int',1,'','see TetGen documentation'],
            ['MaxVolume','maxvolume','float',1,'','see TetGen documentation'],
            ['RemoveSliver','removesliver','int',1,'','see TetGen documentation'],
            ['RegionAttrib','regionattrib','int',1,'','see TetGen documentation'],
            ['Epsilon','epsilon','float',1,'','see TetGen documentation'],
            ['NoMerge','nomerge','int',1,'','see TetGen documentation'],
            ['DetectInter','detectinter','int',1,'','see TetGen documentation'],
            ['CheckClosure','checkclosure','int',1,'','see TetGen documentation'],
            ['Order','order','int',1,'','see TetGen documentation'],
            ['DoCheck','docheck','int',1,'','see TetGen documentation'],
            ['Verbose','verbose','int',1,'','see TetGen documentation'],
            ['UseSizingFunction','usesizingfunction','int',1,'','see TetGen documentation'],
            ['CellEntityIdsArrayName','entityidsarray','str',1,'','name of the array where cell entity ids are stored'],
            ['TetrahedronVolumeArrayName','tetravolumearray','str',1,'','name of the array where volumes of tetrahedra are stored'],
            ['SizingFunctionArrayName','sizingfunctionarray','str',1,'','name of the array where sizing function values are stored'],
            ['OutputSurfaceElements','surfaceelements','int',1,'','toggle output surface elements'],
            ['OutputVolumeElements','volumeelements','int',1,'','toggle output volume elements']
            ])

        self.SetOutputMembers([
            ['Mesh','o','vtkUnstructuredGrid',1,'','the output mesh','vmtkmeshwriter'],
            ['CellEntityIdsArrayName','entityidsarray','str',1,'','name of the array where cell entity ids are stored'],
            ['TetrahedronVolumeArrayName','tetravolumearray','str',1,'','name of the array where volumes of tetrahedra are stored']
            ])

    def Execute(self):

        if self.Mesh == None:
            self.PrintError('Error: No input mesh.')

        if self.GenerateCaps == 1:
            if not ((self.Mesh.IsHomogeneous() == 1) & (self.Mesh.GetCellType(0) == 5)):
                self.PrintError('Error: In order to generate caps, all input mesh elements must be triangles.')
            meshToSurfaceFilter = vtk.vtkGeometryFilter()
            meshToSurfaceFilter.SetInputData(self.Mesh)
            meshToSurfaceFilter.Update()
            cap = vtkvmtk.vtkvmtkSimpleCapPolyData()
            cap.SetInputConnection(meshToSurfaceFilter.GetOutputPort())
            cap.SetCellMarkerArrayName(self.FacetMarkerArrayName)
            cap.Update()
            surfacetomesh = vtkvmtk.vtkvmtkPolyDataToUnstructuredGridFilter()
            surfacetomesh.SetInputConnection(cap.GetOutputPort())
            surfacetomesh.Update()
            self.Mesh = surfacetomesh.GetOutput()

        tetgen = vtkvmtk.vtkvmtkTetGenWrapper()
        tetgen.SetInputData(self.Mesh)
        tetgen.SetPLC(self.PLC)
        tetgen.SetRefine(self.Refine)
        tetgen.SetCoarsen(self.Coarsen)
        tetgen.SetNoBoundarySplit(self.NoBoundarySplit)
        tetgen.SetQuality(self.Quality)
        tetgen.SetMinRatio(self.MinRatio)
        tetgen.SetMinDihedral(self.MinDihedral)
        tetgen.SetMaxDihedral(self.MaxDihedral)
        tetgen.SetVarVolume(self.VarVolume)
        tetgen.SetFixedVolume(self.FixedVolume)
        tetgen.SetMaxVolume(self.MaxVolume)
        tetgen.SetRemoveSliver(self.RemoveSliver)
        tetgen.SetRegionAttrib(self.RegionAttrib)
        tetgen.SetEpsilon(self.Epsilon)
        tetgen.SetNoMerge(self.NoMerge)
        tetgen.SetDetectInter(self.DetectInter)
        tetgen.SetCheckClosure(self.CheckClosure)
        tetgen.SetOrder(self.Order)
        tetgen.SetDoCheck(self.DoCheck)
        tetgen.SetVerbose(self.Verbose)
        tetgen.SetUseSizingFunction(self.UseSizingFunction)
        tetgen.SetCellEntityIdsArrayName(self.CellEntityIdsArrayName)
        tetgen.SetTetrahedronVolumeArrayName(self.TetrahedronVolumeArrayName)
        tetgen.SetSizingFunctionArrayName(self.SizingFunctionArrayName)
        tetgen.SetOutputSurfaceElements(self.OutputSurfaceElements)
        tetgen.SetOutputVolumeElements(self.OutputVolumeElements)
        tetgen.Update()

        self.Mesh = tetgen.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY

__all__ = [
    'vmtk.vmtkboundarylayer2',
    'vmtk.vmtkcenterlinestonumpy',
    'vmtk.vmtkdijkstradistancetopoints',
    'vmtk.vmtkdistancetospheres',
    'vmtk.vmtkentityrenumber',
    'vmtk.vmtkendpointsections',
    'vmtk.vmtkgeodesicsurfaceresolution',
    'vmtk.vmtkimagetonumpy',
    'vmtk.vmtkmeshaddexternallayer',
    'vmtk.vmtkmeshclipcenterlines',
    'vmtk.vmtkmeshmerge',
    'vmtk.vmtkmeshtetrahedralize2',
    'vmtk.vmtkmeshtonumpy',
    'vmtk.vmtkmeshviewer2',
    'vmtk.vmtkmeshwriter2',
    'vmtk.vmtknumpyreader',
    'vmtk.vmtknumpytocenterlines',
    'vmtk.vmtknumpytoimage',
    'vmtk.vmtknumpytomesh',
    'vmtk.vmtknumpytosurface',
    'vmtk.vmtknumpywriter',
    'vmtk.vmtksurfaceextractinnercylinder',
    'vmtk.vmtksurfaceresolution',
    'vmtk.vmtksurfacetonumpy',
    'vmtk.vmtksurfacewriter2',
    'vmtk.vmtkthreshold',
    'vmtk.vmtkpeterresurface',
    'vmtk.vmtkpetergeneratesurface',
    'vmtk.vmtkpetersurfaceclipper'
    ]

for item in __all__:
        exec('from '+item+' import *')



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtksurfaceendclipper.py,v $
## Language:  Python
## Date:      $Date: 2015/12/01 14:45:13 $
## Version:   $Revision: 1.0 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Simone Manini
##       Orobix Srl
## And adapted and modified by
##       Kurt Sansom

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys
from vmtk import vtkvmtk
from vmtk import vmtkrenderer
from vmtk import pypes


## TODO: make SeedSelector a separate pype script to be used in other contexts
class vmtkSeedSelector(object):

    def __init__(self):
        self._Surface = None
        self._SeedIds = None
        self._MainBodySeedIds = vtk.vtkIdList()
        self._SourceSeedIds = vtk.vtkIdList()
        self._TargetSeedIds = vtk.vtkIdList()
        self.PrintError = None
        self.PrintLog = None
        self.InputText = None
        self.OutputText = None
        self.InputInfo = None

    def SetSurface(self,surface):
        self._Surface = surface

    def GetSurface(self):
        return self._Surface

    def GetMainBodySeedIds(self):
        return self._MainBodySeedIds

    def GetSourceSeedIds(self):
        return self._SourceSeedIds

    def GetTargetSeedIds(self):
        return self._TargetSeedIds

    def Execute(self):
        pass


class vmtkPickPointSeedSelector(vmtkSeedSelector):

    def __init__(self):
        vmtkSeedSelector.__init__(self)
        self.PickedSeedIds = vtk.vtkIdList()
        self.PickedSeeds = vtk.vtkPolyData()
        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.Script = None
        self.SetOutlets = 0

    def UndoCallback(self, obj):
        self.InitializeSeeds()
        self.PickedSeeds.Modified()
        self.vmtkRenderer.RenderWindow.Render()

    def PickCallback(self, obj):
        picker = vtk.vtkCellPicker()
        picker.SetTolerance(1E-4 * self._Surface.GetLength())
        eventPosition = self.vmtkRenderer.RenderWindowInteractor.GetEventPosition()
        result = picker.Pick(float(eventPosition[0]),float(eventPosition[1]),0.0,self.vmtkRenderer.Renderer)
        if result == 0:
            return
        pickPosition = picker.GetPickPosition()
        pickedCellPointIds = self._Surface.GetCell(picker.GetCellId()).GetPointIds()
        minDistance = 1E10
        pickedSeedId = -1
        for i in range(pickedCellPointIds.GetNumberOfIds()):
            distance = vtk.vtkMath.Distance2BetweenPoints(pickPosition,self._Surface.GetPoint(pickedCellPointIds.GetId(i)))
            if distance < minDistance:
                minDistance = distance
                pickedSeedId = pickedCellPointIds.GetId(i)
        if pickedSeedId == -1:
            pickedSeedId = pickedCellPointIds.GetId(0)
        self.PickedSeedIds.InsertNextId(pickedSeedId)
        point = self._Surface.GetPoint(pickedSeedId)
        self.PickedSeeds.GetPoints().InsertNextPoint(point)
        self.PickedSeeds.Modified()
        self.vmtkRenderer.RenderWindow.Render()

    def InitializeSeeds(self):
        self.PickedSeedIds.Initialize()
        self.PickedSeeds.Initialize()
        seedPoints = vtk.vtkPoints()
        self.PickedSeeds.SetPoints(seedPoints)

    def Execute(self):

        if (self._Surface == None):
            self.PrintError('vmtkPickPointSeedSelector Error: Surface not set.')
            return

        self._MainBodySeedIds.Initialize()
        self._SourceSeedIds.Initialize()
        self._TargetSeedIds.Initialize()

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.vmtkRenderer.RegisterScript(self.Script)

        glyphs = vtk.vtkGlyph3D()
        glyphSource = vtk.vtkSphereSource()
        glyphs.SetInputData(self.PickedSeeds)
        glyphs.SetSourceConnection(glyphSource.GetOutputPort())
        glyphs.SetScaleModeToDataScalingOff()
        glyphs.SetScaleFactor(self._Surface.GetLength()*0.01)
        glyphMapper = vtk.vtkPolyDataMapper()
        glyphMapper.SetInputConnection(glyphs.GetOutputPort())
        self.SeedActor = vtk.vtkActor()
        self.SeedActor.SetMapper(glyphMapper)
        self.SeedActor.GetProperty().SetColor(1.0,0.0,0.0)
        self.SeedActor.PickableOff()
        self.vmtkRenderer.Renderer.AddActor(self.SeedActor)

        ##self.vmtkRenderer.RenderWindowInteractor.AddObserver("KeyPressEvent", self.KeyPressed)
        self.vmtkRenderer.AddKeyBinding('u','Undo.',self.UndoCallback)
        self.vmtkRenderer.AddKeyBinding('space','Add points.',self.PickCallback)
        surfaceMapper = vtk.vtkPolyDataMapper()
        surfaceMapper.SetInputData(self._Surface)
        surfaceMapper.ScalarVisibilityOff()
        surfaceActor = vtk.vtkActor()
        surfaceActor.SetMapper(surfaceMapper)
        surfaceActor.GetProperty().SetOpacity(1.0)

        self.vmtkRenderer.Renderer.AddActor(surfaceActor)

        self.InputInfo('Please position the mouse and press space to add a point on the main body of the model, \'u\' to undo\n')

        any = 0
        while any == 0:
            self.InitializeSeeds()
            self.vmtkRenderer.Render()
            any = self.PickedSeedIds.GetNumberOfIds()
        self._MainBodySeedIds.DeepCopy(self.PickedSeedIds)

        if( self.SetOutlets ):
            self.InputInfo('Please position the mouse and press space to add point on vessel wall near each Inlet location, \'u\' to undo\n')
        else:
            self.InputInfo('Please position the mouse and press space to add points at clip locations, \'u\' to undo\n')

        any = 0
        while any == 0:
            self.InitializeSeeds()
            self.vmtkRenderer.Render()
            any = self.PickedSeedIds.GetNumberOfIds()
        self._SourceSeedIds.DeepCopy(self.PickedSeedIds)

        if(self.SetOutlets):
            self.InputInfo('Please position the mouse and press space to add point on vessel wall near each Outlet location, \'u\' to undo\n')

            any = 0
            while any == 0:
                self.InitializeSeeds()
                self.vmtkRenderer.Render()
                any = self.PickedSeedIds.GetNumberOfIds()
            self._TargetSeedIds.DeepCopy(self.PickedSeedIds)

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()


class vmtkSurfaceEndClipper(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Centerlines = None
        self.CenterlineNormals = 0
        self.FrenetTangentArrayName = 'FrenetTangent'

        self.Surface = None
        self.vmtkRenderer = None
        self.OwnRenderer = 0
        self.SeedSelector = None
        self.SeedSelectorName = 'pointlist'

        self.SourcePoints = []
        self.TargetPoints = []

        self.CleanOutput = 1

        self.SetScriptName('vmtksurfaceendclipper')
        self.SetScriptDoc('interactively clip a tubular surface with normals estimated at seed locations. It is \
                           recomended that the option to clip based on centerlines normals is enabled to increase \
                           the accuracy of the clipping. If desired, a centerline containing the Frenet Tangent Array \
                           must be fed in along with the surface. This is generated by passing a centerline through the \
                           vmtkcenterlinegeometry script.')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['CenterlineNormals','centerlinenormal','bool',1,'','toggle using centerline tangent for normal estimation'],
            ['Centerlines','centerlines','vtkPolyData',1,'','the input centerlines','vmtksurfacereader'],
            ['FrenetTangentArrayName','frenettangentarray','str',1,'','name of the array where centerline tangent vectors of the Frenet reference system are stored'],
            ['vmtkRenderer','renderer','vmtkRenderer',1,'','external renderer']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface','vmtksurfacewriter'],
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: no Surface.')

        if (self.CenterlineNormals) and (self.Centerlines == None):
            self.PrintError('Error: No input centerlines.')

        if not self.vmtkRenderer:
            self.vmtkRenderer = vmtkrenderer.vmtkRenderer()
            self.vmtkRenderer.Initialize()
            self.OwnRenderer = 1

        self.SeedSelector = vmtkPickPointSeedSelector()
        self.SeedSelector.vmtkRenderer = self.vmtkRenderer
        self.SeedSelector.Script = self

        if (self.CenterlineNormals):
            self.SeedSelector.SetOutlets = 1
        self.SeedSelector.SetSurface(self.Surface)
        self.SeedSelector.InputInfo = self.InputInfo
        self.SeedSelector.InputText = self.InputText
        self.SeedSelector.OutputText = self.OutputText
        self.SeedSelector.PrintError = self.PrintError
        self.SeedSelector.PrintLog = self.PrintLog
        self.SeedSelector.Execute()

        mainBodySeedIds = self.SeedSelector.GetMainBodySeedIds()
        mainBodySeedId = mainBodySeedIds.GetId(0)
        mainBodyPoint = self.Surface.GetPoint(mainBodySeedId)

        clipSeedIds = self.SeedSelector.GetSourceSeedIds()

        surfaceCleaner = vtk.vtkCleanPolyData()
        surfaceCleaner.SetInputData(self.Surface)
        surfaceCleaner.Update()

        surfaceTriangulator = vtk.vtkTriangleFilter()
        surfaceTriangulator.SetInputConnection(surfaceCleaner.GetOutputPort())
        surfaceTriangulator.PassLinesOff()
        surfaceTriangulator.PassVertsOff()
        surfaceTriangulator.Update()

        clippedSurface = surfaceTriangulator.GetOutput()

        n_seeds = clipSeedIds.GetNumberOfIds() # get the number of inlets
        if (self.CenterlineNormals):
            targetSeedIds = self.SeedSelector.GetTargetSeedIds()
            for i in range(targetSeedIds.GetNumberOfIds()):
                clipSeedIds.InsertNextId(targetSeedIds.GetId(i))
            locator_ctr = vtk.vtkPointLocator()
            locator_ctr.SetDataSet(self.Centerlines)
            locator_ctr.BuildLocator()

        for i in range(clipSeedIds.GetNumberOfIds()):

            seedId = clipSeedIds.GetId(i)
            seedPoint = self.Surface.GetPoint(seedId)

            # the frenet tangent must be inverted as it is directed outwards
            # the seam filter requires an inward facing normal
            if (self.CenterlineNormals):
                centerlinePointId = locator_ctr.FindClosestPoint(seedPoint)
                origin = self.Centerlines.GetPoint(centerlinePointId)
                normal = self.Centerlines.GetPointData().GetArray(self.FrenetTangentArrayName).GetTuple(centerlinePointId)
                if( i >= n_seeds ):
                    # invert outward facing normals
                    normal = tuple( -p for p in normal)
            else:
                locator = vtk.vtkPointLocator()
                locator.SetDataSet(clippedSurface)
                locator.BuildLocator()
                seedPointId = locator.FindClosestPoint(seedPoint)

                planeEstimator = vtkvmtk.vtkvmtkPolyDataNormalPlaneEstimator()
                planeEstimator.SetInputData(clippedSurface)
                planeEstimator.SetOriginPointId(seedPointId)
                planeEstimator.Update()
                origin = planeEstimator.GetOrigin()
                normal = planeEstimator.GetNormal()

            plane = vtk.vtkPlane()
            plane.SetOrigin(origin)
            plane.SetNormal(normal)

            seamFilter = vtkvmtk.vtkvmtkTopologicalSeamFilter()
            seamFilter.SetInputData(clippedSurface)
            seamFilter.SetClosestPoint(seedPoint)
            seamFilter.SetSeamScalarsArrayName("SeamScalars")
            seamFilter.SetSeamFunction(plane)

            clipper = vtk.vtkClipPolyData()
            clipper.SetInputConnection(seamFilter.GetOutputPort())
            clipper.GenerateClipScalarsOff()
            clipper.GenerateClippedOutputOn()

            connectivity = vtk.vtkPolyDataConnectivityFilter()
            connectivity.SetInputConnection(clipper.GetOutputPort())
            connectivity.SetExtractionModeToClosestPointRegion()
            connectivity.SetClosestPoint(mainBodyPoint)

            surfaceCleaner = vtk.vtkCleanPolyData()
            surfaceCleaner.SetInputConnection(connectivity.GetOutputPort())
            surfaceCleaner.Update()

            surfaceTriangulator = vtk.vtkTriangleFilter()
            surfaceTriangulator.SetInputConnection(surfaceCleaner.GetOutputPort())
            surfaceTriangulator.PassLinesOff()
            surfaceTriangulator.PassVertsOff()
            surfaceTriangulator.Update()

            #clippedSurface = vtk.vtkPolyData()
            #clippedSurface.DeepCopy(surfaceTriangulator.GetOutput())
            clippedSurface = surfaceTriangulator.GetOutput()

        self.Surface = clippedSurface

        if self.OwnRenderer:
            self.vmtkRenderer.Deallocate()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtkcenterlinesections.py,v $
## Language:  Python
## Date:      $Date: 2006/10/17 15:16:16 $
## Version:   $Revision: 1.1 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
from vmtk import vtkvmtk
import sys

from vmtk import pypes


class vmtkCenterlineSections(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.Centerlines = None
        self.CenterlineSections = None

        self.CenterlineSectionAreaArrayName = 'CenterlineSectionArea'
        self.CenterlineSectionMinSizeArrayName = 'CenterlineSectionMinSize'
        self.CenterlineSectionMaxSizeArrayName = 'CenterlineSectionMaxSize'
        self.CenterlineSectionShapeArrayName = 'CenterlineSectionShape'
        self.CenterlineSectionClosedArrayName = 'CenterlineSectionClosed'

        self.SetScriptName('vmtkcenterlinesections')
        self.SetScriptDoc('compute geometric properties of sections located along centerlines. The script takes in input the surface and the relative centerlines.')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['Centerlines','centerlines','vtkPolyData',1,'','the input centerlines','vmtksurfacereader'],
            ['CenterlineSectionAreaArrayName','branchsectionarea','str',1,'','name of the array where the area of bifurcation sections have to be stored'],
            ['CenterlineSectionMinSizeArrayName','branchsectionminsize','str',1,'','name of the array where the minimum diameter of each section has to be stored'],
            ['CenterlineSectionMaxSizeArrayName','branchsectionmaxsize','str',1,'','name of the array where the maximum diameter of each bifurcation sections has to be stored'],
            ['CenterlineSectionShapeArrayName','centerlinesectionshape','str',1,'','name of the array where the shape index, i.e. the ratio between minimum and maximum diameter, of each bifurcation section has to be stored'],
            ['CenterlineSectionClosedArrayName','branchsectionclosed','str',1,'','name of the array containing 1 if a section is closed and 0 otherwise']
            ])
        self.SetOutputMembers([
            ['CenterlineSections','o','vtkPolyData',1,'','the output sections','vmtksurfacewriter'],
            ['Centerlines','ocenterlines','vtkPolyData',1,'','the output centerlines','vmtksurfacewriter'],
            ['CenterlineSectionAreaArrayName','branchsectionarea','str',1,'','name of the array where the area of bifurcation sections are stored'],
            ['CenterlineSectionMinSizeArrayName','branchsectionminsize','str',1,'','name of the array where the minimum diameter of each section are stored'],
            ['CenterlineSectionMaxSizeArrayName','branchsectionmaxsize','str',1,'','name of the array where the minimum diameter of each bifurcation sections has to be stored'],
            ['CenterlineSectionShapeArrayName','centerlinesectionshape','str',1,'','name of the array where the shape index, i.e. the ratio between minimum and maximum diameter, of each bifurcation section are stored'],
            ['CenterlineSectionClosedArrayName','branchsectionclosed','str',1,'','name of the array containing 1 if a section is closed and 0 otherwise']
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        if self.Centerlines == None:
            self.PrintError('Error: No input centerlines.')

        centerlineSections = vtkvmtk.vtkvmtkPolyDataCenterlineSections()
        centerlineSections.SetInputData(self.Surface)
        centerlineSections.SetCenterlines(self.Centerlines)
        centerlineSections.SetCenterlineSectionAreaArrayName(self.CenterlineSectionAreaArrayName)
        centerlineSections.SetCenterlineSectionMinSizeArrayName(self.CenterlineSectionMinSizeArrayName)
        centerlineSections.SetCenterlineSectionMaxSizeArrayName(self.CenterlineSectionMaxSizeArrayName)
        centerlineSections.SetCenterlineSectionShapeArrayName(self.CenterlineSectionShapeArrayName)
        centerlineSections.SetCenterlineSectionClosedArrayName(self.CenterlineSectionClosedArrayName)
        centerlineSections.Update()

        self.CenterlineSections = centerlineSections.GetOutput()
        self.Centerlines = centerlineSections.GetCenterlines()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtktetringenerator.py,v $
## Language:  Python
## Date:      $Date: 2006/07/27 08:27:40 $
## Version:   $Revision: 1.14 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import sys
import math
import vtk
from vmtk import vtkvmtk

from vmtk import pypes


class SectionProperties:

    def __init__(self):

        self.Radius = 0.0
        self.Normal = [1.0, 0.0, 0.0]
        self.Origin = [0.0, 0.0, 0.0]

        self.NormalizationTransform = None

        self.FlipOutwardNormal = 0

        self.Mesh = None
        self.SectionBoundaryPointIds = None

    def ComputeSectionOrigin(self):
        self.Origin = [0.0, 0.0, 0.0]
        numberOfIds = self.SectionBoundaryPointIds.GetNumberOfIds()
        weightSum = 0.0
        for i in range(numberOfIds):
            point = [0.0,0.0,0.0]
            self.NormalizationTransform.TransformPoint(self.Mesh.GetPoint(self.SectionBoundaryPointIds.GetId(i)),point)
            weight = pow(vtk.vtkMath.Distance2BetweenPoints(self.Mesh.GetPoint(self.SectionBoundaryPointIds.GetId(i)),self.Mesh.GetPoint(self.SectionBoundaryPointIds.GetId((i+1)%numberOfIds))),0.5) + pow(vtk.vtkMath.Distance2BetweenPoints(self.Mesh.GetPoint(self.SectionBoundaryPointIds.GetId(i)),self.Mesh.GetPoint(self.SectionBoundaryPointIds.GetId((i+numberOfIds-1)%numberOfIds))),0.5)
            weightSum += weight
            for j in range(3):
                self.Origin[j] += weight * point[j]
        for j in range(3):
            self.Origin[j] /= weightSum

    def ComputeSectionRadius(self):
        self.Radius = 0.0
        numberOfIds = self.SectionBoundaryPointIds.GetNumberOfIds()
        weightSum = 0.0
        for i in range(numberOfIds):
            point = [0.0,0.0,0.0]
            self.NormalizationTransform.TransformPoint(self.Mesh.GetPoint(self.SectionBoundaryPointIds.GetId(i)),point)
            weight = pow(vtk.vtkMath.Distance2BetweenPoints(self.Mesh.GetPoint(self.SectionBoundaryPointIds.GetId(i)),self.Mesh.GetPoint(self.SectionBoundaryPointIds.GetId((i+1)%numberOfIds))),0.5) + pow(vtk.vtkMath.Distance2BetweenPoints(self.Mesh.GetPoint(self.SectionBoundaryPointIds.GetId(i)),self.Mesh.GetPoint(self.SectionBoundaryPointIds.GetId((i+numberOfIds-1)%numberOfIds))),0.5)
            weightSum += weight
            self.Radius += weight * pow(vtk.vtkMath.Distance2BetweenPoints(self.Origin,point),0.5)
        self.Radius /= weightSum

    def ComputeSectionNormal(self):
        self.Normal = [0.0, 0.0, 0.0]
        numberOfIds = self.SectionBoundaryPointIds.GetNumberOfIds()
        point1 = [0.0,0.0,0.0]
        self.NormalizationTransform.TransformPoint(self.Mesh.GetPoint(self.SectionBoundaryPointIds.GetId(0)),point1)
        vector1 = [point1[0] - self.Origin[0], point1[1] - self.Origin[1], point1[2] - self.Origin[2]]
        vtk.vtkMath.Normalize(vector1)
        vector2 = None
        minAbsDot = 1.0
        for i in range(1,numberOfIds):
            currentPoint = [0.0,0.0,0.0]
            self.NormalizationTransform.TransformPoint(self.Mesh.GetPoint(self.SectionBoundaryPointIds.GetId(i)),currentPoint)

            currentVector = [currentPoint[0] - self.Origin[0], currentPoint[1] - self.Origin[1], currentPoint[2] - self.Origin[2]]
            vtk.vtkMath.Normalize(currentVector)
            absDot = abs(vtk.vtkMath.Dot(vector1,currentVector))
            if (absDot<minAbsDot):
                minAbsDot = absDot
                vector2 = currentVector
        self.Normal = [0.0,0.0,0.0]
        vtk.vtkMath.Cross(vector1,vector2,self.Normal)
        vtk.vtkMath.Normalize(self.Normal)

    def FlipNormal(self):
        for j in range(3):
            if (self.Normal[j] != 0.0):
                self.Normal[j] *= -1.0

    def OrientNormalOutward(self):
        id = self.SectionBoundaryPointIds.GetId(0)
        point = [0.0,0.0,0.0]
        self.NormalizationTransform.TransformPoint(self.Mesh.GetPoint(id),point)
        cellIds = vtk.vtkIdList()
        self.Mesh.GetPointCells(id,cellIds)
        cell = self.Mesh.GetCell(cellIds.GetId(0))
        maxAbsDot = 0.0
        farVector = None
        for i in range(cell.GetNumberOfPoints()):
            currentPoint = [0.0,0.0,0.0]
            self.NormalizationTransform.TransformPoint(cell.GetPoints().GetPoint(i),currentPoint)

            currentVector = [point[0] - currentPoint[0], point[1] - currentPoint[1], point[2] - currentPoint[2]]
            absDot = abs(vtk.vtkMath.Dot(self.Normal,currentVector))
            if (absDot>maxAbsDot):
                maxAbsDot = absDot
                farVector = currentVector
        dot = vtk.vtkMath.Dot(self.Normal,farVector)
        if (dot < 0.0):
            self.FlipNormal()

    def Execute(self):

        if self.Mesh == None:
            return

        if self.SectionBoundaryPointIds == None:
            return

        self.ComputeSectionOrigin()
        self.ComputeSectionRadius()
        self.ComputeSectionNormal()
        self.OrientNormalOutward()

        if (self.FlipOutwardNormal == 1):
            self.FlipNormal()


class vmtkTetrInGenerator(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.OutputFileName = ''
        self.Mesh = None
        self.NormalizationTransform = None

        self.NormalizationRadius = 1.0

        self.NormalizationEntity = ''
        self.InletEntities = []
        self.ReverseInletEntities = []
        self.OutletEntity = ''
        self.WallEntity = ''
        self.HistoryEntity = ''

        self.UseCellDefinedEntities = 1
        self.CellEntityIdsArrayName = 'CellEntityIds'
        self.NormalizationEntityId = -1
        self.InletEntityIds = []
        self.OutletEntityId = -1
        self.WallEntityId = -1
        self.HistoryEntityId = -1

        self.ReverseInlets = []

        self.TimeStepsOnly = 0

        self.StartTime = 0.0
        self.EndTime = 1.0
        self.NumberOfTimeSteps = 0
        self.NumberOfDumps = 0
        self.DumpFlag = '11000'

        self.WriteWNodeSection = 1
        self.WriteWElemSection = 1

##             ['NormalizationEntity','normalizationentity','str',1],
##             ['InletEntities','inletentities','str',-1],
##             ['OutletEntity','outletentity','str',1],
##             ['WallEntity','wallentity','str',1],
##             ['HistoryEntity','historyentity','str',1],
##             ['UseCellDefinedEntities','celldefinedentities','int',1],

        self.SetScriptName('vmtktetringenerator')
        self.SetScriptDoc('generate input files for the newtetr CFD solver')
        self.SetInputMembers([
            ['Mesh','i','vtkUnstructuredGrid',1,'','the input mesh','vmtkmeshreader'],
            ['OutputFileName','ofile','str',1,'','output file name'],
            ['TimeStepsOnly','timestepsonly','bool',1,'','only generate the $time section of the tetr.in file'],
            ['CellEntityIdsArrayName','entityidsarray','str',1,'','name of the array where entity ids relative to cells are stored'],
            ['NormalizationEntityId','normalizationid','int',1,'','id of the entity relative to which the mesh has to be normalized'],
            ['NormalizationRadius','normalizationradius','float',1,'(0.0,)','explicit value of the radius relative to which the mesh has to be normalized (to be used when -normalizationid is not used)'],
            ['InletEntityIds','inletids','int',-1,'','id of inlet entities'],
            ['OutletEntityId','outletid','int',1,'','id of the outlet entity'],
            ['WallEntityId','wallid','int',1,'','id of the wall entity'],
            ['HistoryEntityId','historyid','int',1,'','id of the entity whose history has to be stored'],
            ['ReverseInlets','reverseinlets','bool',-1,'','toggle reversal of the nth inlet entity normal'],
            ['StartTime','starttime','float',1,'','simulation normalized start time'],
            ['EndTime','endtime','float',1,'','simulation normalized end time'],
            ['NumberOfTimeSteps','timesteps','int',1,'(0,)','number of time steps between start and end time'],
            ['NumberOfDumps','dumps','int',1,'(0,)','number of solution dumps between start and end time'],
            ['DumpFlag','dumpflag','str',1,'','flag identifying solution dump (e.g. 11000 = tetr.pres and tetr.vel)'],
            ['WriteWNodeSection','wnodesection','bool',1,'','append $wnode section to .in file for subsequent wall shear stress computation'],
            ['WriteWElemSection','welemsection','bool',1,'','append $welem section to .in file for subsequent wall shear stress computation']
            ])
        self.SetOutputMembers([])

    def ComputeBarycenter(self,pointIds):
        barycenter = [0.0, 0.0, 0.0]
        numberOfIds = pointIds.GetNumberOfIds()
        for i in range(numberOfIds):
            point = self.Mesh.GetPoint(pointIds.GetId(i))
            for j in range(3):
                barycenter[j] += point[j]
        for j in range(3):
            barycenter[j] /= numberOfIds
        return barycenter

    def ComputeAngle(self,origin,point0,point1):
        vector0 = [
            point0[0] - origin[0],
            point0[1] - origin[1],
            point0[2] - origin[2]]
        vector1 = [
            point1[0] - origin[0],
            point1[1] - origin[1],
            point1[2] - origin[2]]
        vtk.vtkMath.Normalize(vector0)
        vtk.vtkMath.Normalize(vector1)
        sum = [
            vector0[0] + vector1[0],
            vector0[1] + vector1[1],
            vector0[2] + vector1[2]]
        difference = [
            vector0[0] - vector1[0],
            vector0[1] - vector1[1],
            vector0[2] - vector1[2]]
        sumNorm = vtk.vtkMath.Norm(sum)
        differenceNorm = vtk.vtkMath.Norm(difference)
        angle = 2.0 * math.atan2(differenceNorm,sumNorm) + math.pi
        return angle

    def SortBoundaryPoints(self,sectionBoundaryPointIds):
        barycenter = self.ComputeBarycenter(sectionBoundaryPointIds)
        numberOfIds = sectionBoundaryPointIds.GetNumberOfIds()
        for i in range(0,numberOfIds-1):
            minAngleId = -1
            minAngle = 2.0 * math.pi
            point0 = self.Mesh.GetPoint(sectionBoundaryPointIds.GetId(i))
            for j in range(i+1,numberOfIds):
                point1 = self.Mesh.GetPoint(sectionBoundaryPointIds.GetId(j))
                angle = self.ComputeAngle(barycenter,point0,point1)
                if angle < minAngle:
                    minAngleId = j
                    minAngle = angle
            tempId = sectionBoundaryPointIds.GetId(i+1)
            sectionBoundaryPointIds.SetId(i+1,sectionBoundaryPointIds.GetId(minAngleId))
            sectionBoundaryPointIds.SetId(minAngleId,tempId)

    def GetSectionBoundaryPointIds(self,sectionEntity):

        if (self.WallEntity == ''):
            self.PrintLog('Cannot compute section properties if WallEntity is undefined.')
            return None

        sectionEntityArray = self.Mesh.GetPointData().GetArray(sectionEntity)
        wallEntityArray = self.Mesh.GetPointData().GetArray(self.WallEntity)

        sectionBoundaryPointIds = vtk.vtkIdList()

        for i in range(self.Mesh.GetNumberOfPoints()):
            if (sectionEntityArray.GetComponent(i,0) != 0) & (wallEntityArray.GetComponent(i,0) != 0):
                sectionBoundaryPointIds.InsertNextId(i)

        self.SortBoundaryPoints(sectionBoundaryPointIds)

        return sectionBoundaryPointIds

    def GenerateTetrInFile(self):

        self.PrintLog('Generating Tetr .in file.')

        f=open(self.OutputFileName, 'w')

        line = '$title' + '\n'
        f.write(line)
        line = self.OutputFileName + '\n'
        f.write(line)
        line = '\n'
        f.write(line)

        line = '$compress' + '\n'
        f.write(line)
        line = 'gzip -f' + '\n'
        f.write(line)
        line = '\n'
        f.write(line)

        self.NormalizationTransform.Identity()

        if (self.NormalizationEntity != ''):
            sectionBoundaryPointIds = self.GetSectionBoundaryPointIds(self.NormalizationEntity)
            sectionProperties = SectionProperties()
            sectionProperties.Mesh = self.Mesh
            sectionProperties.NormalizationTransform = self.NormalizationTransform
            sectionProperties.SectionBoundaryPointIds = sectionBoundaryPointIds
            sectionProperties.Execute()
            self.NormalizationRadius = sectionProperties.Radius

        if (self.NormalizationRadius != 0.0) & (self.NormalizationRadius != 1.0):
            self.NormalizationTransform.Scale(1.0/self.NormalizationRadius,1.0/self.NormalizationRadius,1.0/self.NormalizationRadius)

        line = '$radius' + '\n'
        f.write(line)
        line = str(self.NormalizationRadius) + '\n'
        f.write(line)
        line = '\n'
        f.write(line)

        line = '$viscosity' + '\n'
        f.write(line)
        line = str(0.035) + '\n'
        f.write(line)
        line = '\n'
        f.write(line)

        line = '$density' + '\n'
        f.write(line)
        line = str(1.06) + '\n'
        f.write(line)
        line = '\n'
        f.write(line)

        line = '$red' + '\n'
        f.write(line)
        line = str(0) + '\n'
        f.write(line)
        line = '\n'
        f.write(line)

        line = '$alpha' + '\n'
        f.write(line)
        line = str(0) + '\n'
        f.write(line)
        line = '\n'
        f.write(line)

        line = '$nsolve' + '\n'
        f.write(line)
        line = '22' + '\n'
        f.write(line)
        line = '\n'
        f.write(line)

        line = '$Uzawa_PC' + '\n'
        f.write(line)
        line = '1' + '\n'
        f.write(line)
        line = '\n'
        f.write(line)

        line = '$node' + '\n'
        f.write(line)
        line = str(self.Mesh.GetNumberOfPoints()) + '\n'
        f.write(line)
        for i in range(self.Mesh.GetNumberOfPoints()):
            point = [0.0,0.0,0.0]
            self.NormalizationTransform.TransformPoint(self.Mesh.GetPoint(i),point)
            line = str(i+1) + ' ' + str(point[0]) + ' ' + str(point[1]) + ' ' + str(point[2]) + '\n'
            f.write(line)

        line = '\n'
        f.write(line)

        line = '$elem' + '\n'
        f.write(line)
        quadratidTetraCellType = 24
        quadraticTetraCellIds = vtk.vtkIdTypeArray()
        self.Mesh.GetIdsOfCellsOfType(quadratidTetraCellType,quadraticTetraCellIds)
        line = str(quadraticTetraCellIds.GetNumberOfTuples()) + '\n'
        f.write(line)
        for i in range(quadraticTetraCellIds.GetNumberOfTuples()):
            line = str(i+1) + ' '
            cell = self.Mesh.GetCell(quadraticTetraCellIds.GetValue(i))
            line += str(cell.GetPointId(0)+1) + ' '
            line += str(cell.GetPointId(4)+1) + ' '
            line += str(cell.GetPointId(1)+1) + ' '
            line += str(cell.GetPointId(5)+1) + ' '
            line += str(cell.GetPointId(2)+1) + ' '
            line += str(cell.GetPointId(6)+1) + ' '
            line += str(cell.GetPointId(7)+1) + ' '
            line += str(cell.GetPointId(8)+1) + ' '
            line += str(cell.GetPointId(9)+1) + ' '
            line += str(cell.GetPointId(3)+1) + ' '
            line += '\n'
            f.write(line)

#        inletEntities = self.InletEntities
#        reversedInlets = self.ReverseInlets

#        inletEntitiesReversed = zip(inletEntities,reversedInlets)

#        for inletEntityReversed in inletEntitiesReversed:
#            inletEntity = inletEntityReversed[0]
#            reversedInlet = inletEntityReversed[1]
#            if (inletEntity == ''):
#                continue
        for inletEntity in self.InletEntities:
            reversedInlet = inletEntity in self.ReverseInletEntities
            line = '\n'
            f.write(line)
            line = '$binlet' + ' (' + inletEntity + ') ' + '\n'
            f.write(line)
            entityArray = self.Mesh.GetPointData().GetArray(inletEntity)
            numberOfEntityNodes = 0
            for i in range(entityArray.GetNumberOfTuples()):
                if (entityArray.GetComponent(i,0) != 1.0):
                    continue
                numberOfEntityNodes += 1
            line = str(numberOfEntityNodes) + '\n'
            f.write(line)
            for i in range(entityArray.GetNumberOfTuples()):
                if (entityArray.GetComponent(i,0) != 1.0):
                    continue
                line = str(i+1) + '\n'
                f.write(line)

            sectionBoundaryPointIds = self.GetSectionBoundaryPointIds(inletEntity)
            sectionProperties = SectionProperties()
            sectionProperties.Mesh = self.Mesh
            sectionProperties.NormalizationTransform = self.NormalizationTransform
            sectionProperties.SectionBoundaryPointIds = sectionBoundaryPointIds
            if not reversedInlet:
                sectionProperties.FlipOutwardNormal = 1
            else:
                sectionProperties.FlipOutwardNormal = 0
            sectionProperties.Execute()
            line = str(sectionProperties.Radius) + '\n'
            f.write(line)
            line = str(sectionProperties.Origin[0]) + ' ' + str(sectionProperties.Origin[1]) + ' ' + str(sectionProperties.Origin[2]) + '\n'
            f.write(line)
            line = str(sectionProperties.Normal[0]) + ' ' + str(sectionProperties.Normal[1]) + ' ' + str(sectionProperties.Normal[2]) + '\n'
            f.write(line)
            #TODO: for every inlet insert fourier coefficients given points of waveform (spline-interpolate points beforehands to get them equispaced).

        if (self.OutletEntity != ''):
            line = '\n'
            f.write(line)
            line = '$boutlet' + ' (' + self.OutletEntity + ') ' + '\n'
            f.write(line)
            entityArray = self.Mesh.GetPointData().GetArray(self.OutletEntity)
            numberOfEntityNodes = 0
            for i in range(entityArray.GetNumberOfTuples()):
                if (entityArray.GetComponent(i,0) != 1.0):
                    continue
                numberOfEntityNodes += 1
            line = str(numberOfEntityNodes) + '\n'
            f.write(line)
            for i in range(entityArray.GetNumberOfTuples()):
                if (entityArray.GetComponent(i,0) != 1.0):
                    continue
                line = str(i+1) + '\n'
                f.write(line)
            sectionBoundaryPointIds = self.GetSectionBoundaryPointIds(self.OutletEntity)
            sectionProperties = SectionProperties()
            sectionProperties.Mesh = self.Mesh
            sectionProperties.NormalizationTransform = self.NormalizationTransform
            sectionProperties.SectionBoundaryPointIds = sectionBoundaryPointIds
            sectionProperties.FlipOutwardNormal = 0
            sectionProperties.Execute()
            line = str(sectionProperties.Radius) + '\n'
            f.write(line)
            line = str(sectionProperties.Origin[0]) + ' ' + str(sectionProperties.Origin[1]) + ' ' + str(sectionProperties.Origin[2]) + '\n'
            f.write(line)
            line = str(sectionProperties.Normal[0]) + ' ' + str(sectionProperties.Normal[1]) + ' ' + str(sectionProperties.Normal[2]) + '\n'
            f.write(line)

        if (self.WallEntity != ''):
            line = '\n'
            f.write(line)
            line = '$bwall' + '\n'
            f.write(line)
            entityArray = self.Mesh.GetPointData().GetArray(self.WallEntity)
            numberOfEntityNodes = 0
            for i in range(entityArray.GetNumberOfTuples()):
                if (entityArray.GetComponent(i,0) != 1.0):
                    continue
                numberOfEntityNodes += 1
            line = str(numberOfEntityNodes) + '\n'
            f.write(line)
            for i in range(entityArray.GetNumberOfTuples()):
                if (entityArray.GetComponent(i,0) != 1.0):
                    continue
                line = str(i+1) + '\n'
                f.write(line)

            wallPointIdsMap = vtk.vtkIdList()
            if (self.WriteWNodeSection == 1):
                line = '\n'
                f.write(line)
                line = '$wnode' + '\n'
                f.write(line)
                line = str(numberOfEntityNodes) + '\n'
                f.write(line)
                count = 0
                wallPointIdsMap.SetNumberOfIds(entityArray.GetNumberOfTuples())
                extractSurface = vtk.vtkGeometryFilter()
                extractSurface.SetInputData(self.Mesh)
                extractSurface.Update()
                # Since VTK 9.4, vtkPolyDataNormals passes pre-existing normals
                # through unchanged; strip normals inherited from the mesh so
                # they are always recomputed.
                extractedSurface = vtk.vtkPolyData()
                extractedSurface.ShallowCopy(extractSurface.GetOutput())
                extractedSurface.GetPointData().SetNormals(None)
                extractedSurface.GetCellData().SetNormals(None)
                normalsFilter = vtk.vtkPolyDataNormals()
                normalsFilter.SetInputData(extractedSurface)
                normalsFilter.AutoOrientNormalsOn()
                normalsFilter.ConsistencyOn()
                normalsFilter.Update()
                normalsSurface = normalsFilter.GetOutput()
                locator = vtk.vtkMergePoints()
                locator.SetDataSet(normalsSurface)
                locator.BuildLocator()
                for i in range(entityArray.GetNumberOfTuples()):
                    if (entityArray.GetComponent(i,0) != 1.0):
                        wallPointIdsMap.SetId(i,-1)
                        continue
                    wallPointIdsMap.SetId(i,count)
                    point = self.Mesh.GetPoint(i)
                    surfacePointId = locator.FindClosestPoint(point)
                    normal = normalsSurface.GetPointData().GetNormals().GetTuple3(surfacePointId)
                    line = str(count+1) + ' ' + str(i+1) + ' ' + str(normal[0]) + ' ' + str(normal[1]) + ' ' + str(normal[2]) + '\n'
                    f.write(line)
                    count += 1

            if (self.WriteWElemSection == 1):
                line = '\n'
                f.write(line)
                line = '$welem' + '\n'
                f.write(line)
                dataArray = vtk.vtkIntArray()
                dataArray.SetNumberOfComponents(8)
                for i in range(self.Mesh.GetNumberOfCells()):
                    qtetra = self.Mesh.GetCell(i)
                    for j in range(qtetra.GetNumberOfFaces()):
                        face = qtetra.GetFace(j)
                        isBoundaryFace = 1
                        for k in range(face.GetNumberOfPoints()):
                            if (entityArray.GetComponent(face.GetPointId(k),0) != 1.0):
                                isBoundaryFace = 0
                                break
                        if (isBoundaryFace == 0):
                            continue
                        dataArray.InsertNextValue(i)
                        dataArray.InsertNextValue(j)
                        for k in range(face.GetNumberOfPoints()):
                            dataArray.InsertNextValue(face.GetPointId(k))
                line = str(dataArray.GetNumberOfTuples()) + '\n'
                f.write(line)
                for i in range(dataArray.GetNumberOfTuples()):
                    line = str(i+1) + ' '
                    line += str(int(dataArray.GetComponent(i,0))+1) + ' '
                    faceId = int(dataArray.GetComponent(i,1))
                    newTetrFaceIdsMap = [2, 4, 3, 1]
                    line += str(newTetrFaceIdsMap[faceId]) + ' '
                    for j in range(2,dataArray.GetNumberOfComponents()):
                        line += str(wallPointIdsMap.GetId(int(dataArray.GetComponent(i,j)))+1) + ' '
                    line += '\n'
                    f.write(line)

        if (self.HistoryEntity != ''):
            line = '\n'
            f.write(line)
            line = '$history' + '\n'
            f.write(line)
            entityArray = self.Mesh.GetPointData().GetArray(self.HistoryEntity)
            numberOfEntityNodes = 0
            for i in range(entityArray.GetNumberOfTuples()):
                if (entityArray.GetComponent(i,0) != 1.0):
                    continue
                numberOfEntityNodes += 1
            line = str(numberOfEntityNodes) + '\n'
            f.write(line)
            for i in range(entityArray.GetNumberOfTuples()):
                if (entityArray.GetComponent(i,0) != 1.0):
                    continue
                line = str(i+1) + '\n'
                f.write(line)

        self.WriteTimeSteps(f)

    def GenerateTimeStepsFile(self):

        self.PrintLog('Generating $time section.')
        f=open(self.OutputFileName, 'w')
        self.WriteTimeSteps(f)

    def WriteTimeSteps(self,f):

        if (self.NumberOfTimeSteps > 0):
            line = '\n'
            f.write(line)
            line = '$time' + '\n'
            f.write(line)
            line = str(self.NumberOfTimeSteps) + '\n'
            f.write(line)
            line = str(self.StartTime) + '\n'
            f.write(line)
            timeStep = (self.EndTime - self.StartTime) / float(self.NumberOfTimeSteps)
            for i in range(1,self.NumberOfTimeSteps+1):
                time = self.StartTime + float(i) * timeStep
                dumpFlag = '0000'
                if (i % (self.NumberOfTimeSteps / self.NumberOfDumps) == 0):
                    dumpFlag = self.DumpFlag
                nsub = 0
                line = str(time) + ' ' + dumpFlag + ' ' + str(nsub) + '\n'
                f.write(line)

    def BuildPointEntityArray(self,entityId,pointEntityArray):

        cellEntityArray = self.Mesh.GetCellData().GetArray(self.CellEntityIdsArrayName)
        pointEntityArray.SetNumberOfTuples(self.Mesh.GetNumberOfPoints())
        pointEntityArray.FillComponent(0,0.0)

        for i in range(self.Mesh.GetNumberOfCells()):
            cellEntityId = int(cellEntityArray.GetComponent(i,0))
            if cellEntityId != entityId:
                continue
            cell = self.Mesh.GetCell(i)
            for j in range(cell.GetNumberOfPoints()):
                pointEntityArray.SetComponent(cell.GetPointId(j),0,1)

    def Execute(self):

        if (self.OutputFileName == ''):
            self.PrintError('Error: no OutputFileName.')

        if self.TimeStepsOnly:
            self.GenerateTimeStepsFile()
            return

        if self.Mesh == None:
            self.PrintError('Error: no Mesh.')

        self.NormalizationTransform = vtk.vtkTransform()

        if self.UseCellDefinedEntities == 1:

            entityIds = []
            entityNames = []

            if self.NormalizationEntityId != -1:
                entityIds.append(self.NormalizationEntityId)
                entityNames.append('NormalizationEntity')
                self.NormalizationEntity = 'NormalizationEntity'
            for inletEntityId in self.InletEntityIds:
                entityIds.append(inletEntityId)
                entityNames.append('InletEntity'+str(inletEntityId))
                self.InletEntities.append('InletEntity'+str(inletEntityId))
            for reverseInletEntityId in self.ReverseInlets:
                self.ReverseInletEntities.append('InletEntity'+str(reverseInletEntityId))
            if self.OutletEntityId != -1:
                entityIds.append(self.OutletEntityId)
                entityNames.append('OutletEntity')
                self.OutletEntity = 'OutletEntity'
            if self.WallEntityId != -1:
                entityIds.append(self.WallEntityId)
                entityNames.append('WallEntity')
                self.WallEntity = 'WallEntity'
            if self.HistoryEntityId != -1:
                entityIds.append(self.HistoryEntityId)
                entityNames.append('HistoryEntity')
                self.HistoryEntity = 'HistoryEntity'

            for i in range(len(entityIds)):
                entityId = entityIds[i]
                entityName = entityNames[i]
                pointEntityArray = vtk.vtkIntArray()
                pointEntityArray.SetName(entityName)
                self.BuildPointEntityArray(entityId,pointEntityArray)
                self.Mesh.GetPointData().AddArray(pointEntityArray)

        self.GenerateTetrInFile()


if __name__=='__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      February 7, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkbifurcationvectors as bifurcationvectors
import vmtk.vmtksurfacereader as surfacereader
import os
from vtk.numpy_interface import dataset_adapter as dsa
import numpy as np


@pytest.fixture(scope='module')
def reference_system(input_datadir):
    reader = surfacereader.vmtkSurfaceReader()
    reader.InputFileName = os.path.join(input_datadir, 'aorta-centerline-referencesystem.vtp')
    reader.Execute()
    return reader.Surface


@pytest.fixture(scope='module')
def bifurcation_vectors(reference_system, aorta_centerline_branches):
    vectors = bifurcationvectors.vmtkBifurcationVectors()
    vectors.Centerlines = aorta_centerline_branches
    vectors.ReferenceSystems = reference_system
    vectors.RadiusArrayName = "MaximumInscribedSphereRadius"
    vectors.GroupIdsArrayName = "GroupIds"
    vectors.CenterlineIdsArrayName = "CenterlineIds"
    vectors.TractIdsArrayName = "TractIds"
    vectors.BlankingArrayName = "Blanking"
    vectors.ReferenceSystemsNormalArrayName = "Normal"
    vectors.ReferenceSystemsUpNormalArrayName = "UpNormal"
    vectors.NormalizeBifurcationVectors = 0
    vectors.Execute()

    return vectors.BifurcationVectors


def test_number_of_points_is_three(bifurcation_vectors):
    assert bifurcation_vectors.GetNumberOfPoints() == 3


def test_number_of_point_arrays_is_eight(bifurcation_vectors):
    assert bifurcation_vectors.GetPointData().GetNumberOfArrays() == 8


@pytest.mark.parametrize("arraynum,expectedname", [
    (0, "BifurcationVectors"),
    (1, "InPlaneBifurcationVectors"),
    (2, "OutOfPlaneBifurcationVectors"),
    (3, "InPlaneBifurcationVectorAngles"),
    (4, "OutOfPlaneBifurcationVectorAngles"),
    (5, "BifurcationVectorsOrientation"),
    (6, "GroupIds"),
    (7, "BifurcationGroupIds"),
])
def test_point_data_arrays_names_align_with_expected(bifurcation_vectors, arraynum, expectedname):
    pointData = bifurcation_vectors.GetPointData()
    arrayname = pointData.GetArrayName(arraynum)
    assert arrayname == expectedname


@pytest.mark.parametrize("arraynum,expectednum", [
    (0, 3),
    (1, 3),
    (2, 3),
    (3, 1),
    (4, 1),
    (5, 1),
    (6, 1),
    (7, 1),
])
def test_point_data_arrays_number_of_components_align_with_expected(bifurcation_vectors, arraynum, expectednum):
    pointData = bifurcation_vectors.GetPointData()
    numberofcomponents = pointData.GetArray(arraynum).GetNumberOfComponents()
    assert numberofcomponents == expectednum


def test_point_locations_are_where_expected(bifurcation_vectors):
    wrapped = dsa.WrapDataObject(bifurcation_vectors)
    expectedPoints = np.array([[ 221.4466095 ,  147.7495575 ,   23.43517685],
                               [ 225.06373596,  134.70544434,   23.55709076],
                               [ 218.53140259,  134.76171875,   24.90955925]])

    assert np.allclose(wrapped.Points, expectedPoints) == True


def test_bifurcation_vector_values_align_with_expected(bifurcation_vectors):
    wrapped = dsa.WrapDataObject(bifurcation_vectors)
    expectedPoints = np.array([[ 0.24890407, -6.95878637,  0.45230165],
                               [ 2.08810306, -4.00949241,  0.14137213],
                               [-2.17909268, -4.12408806,  0.56015663]])

    assert np.allclose(wrapped.PointData['BifurcationVectors'], expectedPoints) == True


def test_inplane_bifurcation_vector_values_align_with_expected(bifurcation_vectors):
    wrapped = dsa.WrapDataObject(bifurcation_vectors)
    expectedPoints = np.array([[ 0.23626338, -6.96323822,  0.37824108],
                               [ 2.04566927, -4.02443691, -0.10724324],
                               [-2.16923032, -4.1206147 ,  0.61793922]])

    assert np.allclose(wrapped.PointData['InPlaneBifurcationVectors'], expectedPoints) == True


def test_outofplane_bifurcation_vector_values_align_with_expected(bifurcation_vectors):
    wrapped = dsa.WrapDataObject(bifurcation_vectors)
    expectedPoints = np.array([[ 0.01264069,  0.00445185,  0.07406057],
                               [ 0.04243379,  0.0149445 ,  0.24861537],
                               [-0.00986236, -0.00347336, -0.05778259]])

    assert np.allclose(wrapped.PointData['OutOfPlaneBifurcationVectors'], expectedPoints) == True


def test_inplane_bifurcation_vector_angle_values_align_with_expected(bifurcation_vectors):
    wrapped = dsa.WrapDataObject(bifurcation_vectors)
    expectedPoints = np.array([ 0.03805914,  0.48117113, -0.48381398])

    assert np.allclose(wrapped.PointData['InPlaneBifurcationVectorAngles'], expectedPoints) == True


def test_outofplane_bifurcation_vector_angle_values_align_with_expected(bifurcation_vectors):
    wrapped = dsa.WrapDataObject(bifurcation_vectors)
    expectedPoints = np.array([-0.01078616, -0.05589054,  0.01249973])

    assert np.allclose(wrapped.PointData['OutOfPlaneBifurcationVectorAngles'], expectedPoints) == True


def test_bifurcation_vector_orientation_values_align_with_expected(bifurcation_vectors):
    wrapped = dsa.WrapDataObject(bifurcation_vectors)
    expectedPoints = np.array([0, 1, 1])

    assert np.allclose(wrapped.PointData['BifurcationVectorsOrientation'], expectedPoints) == True


def test_groupid_values_align_with_expected(bifurcation_vectors):
    wrapped = dsa.WrapDataObject(bifurcation_vectors)
    expectedPoints = np.array([0, 2, 3])

    assert np.allclose(wrapped.PointData['GroupIds'], expectedPoints) == True


def test_bifurcation_groupid_values_align_with_expected(bifurcation_vectors):
    wrapped = dsa.WrapDataObject(bifurcation_vectors)
    expectedPoints = np.array([1, 1, 1])

    assert np.allclose(wrapped.PointData['BifurcationGroupIds'], expectedPoints) == True



#!/usr/bin/env python

## Program:   VMTK
## Module:    $RCSfile: vmtklineresampling.py,v $
## Language:  Python
## Date:      $Date: 2005/09/14 09:49:59 $
## Version:   $Revision: 1.4 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys

from vmtk import pypes


class vmtkLineResampling(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None

        self.Length = 0.0;

        self.SetScriptName('vmtklineresampling')
        self.SetScriptDoc('resample input lines with a spline filter')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface containing vtkPolyLine cells','vmtksurfacereader'],
            ['Length','length','float',1,'(0.0,)','length of the resampling interval']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface containing the resampled vtkPolyLine cells','vmtksurfacewriter']
            ])

    def Execute(self):

        if self.Surface == None:
            self.PrintError('Error: No input surface.')

        cleaner = vtk.vtkCleanPolyData()
        cleaner.SetInputData(self.Surface)
        cleaner.Update()

        if self.Length == 0.0:
            self.Length = cleaner.GetOutput().GetLength() / 100.0

        splineFilter = vtk.vtkSplineFilter()
        splineFilter.SetInputConnection(cleaner.GetOutputPort())
        splineFilter.SetSubdivideToLength()
        splineFilter.SetLength(self.Length)
        splineFilter.Update()

        self.Surface = splineFilter.GetOutput()


if __name__=='__main__':

    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program:   VMTK
## Module:    $RCSfile: vmtksurfacearraysmoothing.py,v $
## Language:  Python
## Date:      $Date: 2014/10/24 12:35:13 $
## Version:   $Revision: 1.10 $

##   Copyright (c) Luca Antiga, David Steinman. All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this class was contributed by
##       Elena Faggiano (elena.faggiano@gmail.com)
##       Politecnico di Milano

from __future__ import absolute_import #NEEDS TO STAY AS TOP LEVEL MODULE FOR Py2-3 COMPATIBILITY
import vtk
import sys
from vmtk import pypes
import math


class vmtkSurfaceArraySmoothing(pypes.pypeScript):

    def __init__(self):

        pypes.pypeScript.__init__(self)

        self.Surface = None
        self.SurfaceArrayName = ''
        self.Connexity = 1
        self.Relaxation = 1.0
        self.Iterations = 1

        self.SetScriptName('vmtksurfacearraysmoothing')
        self.SetScriptDoc('Perform smoothing of the point array defined on the surface')
        self.SetInputMembers([
            ['Surface','i','vtkPolyData',1,'','the input surface','vmtksurfacereader'],
            ['SurfaceArrayName','surfacearray','str',1],
            ['Connexity','connexity','int',1,'(1,2)','patch connexity considered in the smoothing procedure'],
            ['Relaxation','relaxation','float',1,'(0.0,1.0)','relaxation factor'],
            ['Iterations','iterations','int',1,'','number of smoothing iterations']
            ])
        self.SetOutputMembers([
            ['Surface','o','vtkPolyData',1,'','the output surface with the smoothed array','vmtksurfacewriter'],
            ])

    def Execute(self):

        if (self.Surface == None):
            self.PrintError('Error: no Surface.')

        if (self.Surface.GetPointData().GetArray(self.SurfaceArrayName) == None):
            self.PrintError('Error: no array with name specified')
        else:
            array = self.Surface.GetPointData().GetArray(self.SurfaceArrayName)

        surface = self.Surface
        extractEdges = vtk.vtkExtractEdges()
        extractEdges.SetInputData(surface)
        extractEdges.Update()
        surfEdges = extractEdges.GetOutput()

        if self.Connexity == 1:
            for n in range (self.Iterations):
                for i in range (surfEdges.GetNumberOfPoints()):
                    cells = vtk.vtkIdList()
                    surfEdges.GetPointCells(i,cells)
                    vval = 0
                    ddd = 0
                    d = 0
                    N = 0
                    for j in range (cells.GetNumberOfIds()):
                        points = vtk.vtkIdList()
                        surfEdges.GetCellPoints(cells.GetId(j),points)
                        for k in range (points.GetNumberOfIds()):
                            if points.GetId(k) != i:
                                d = math.sqrt(vtk.vtkMath.Distance2BetweenPoints(surface.GetPoint(i),surface.GetPoint(points.GetId(k))))
                                dd = 1/d
                                val = array.GetComponent(points.GetId(k),0)#*dd
                                N = N+1
                                vval = vval + val
                                ddd = ddd + dd
                    val = array.GetComponent(i,0)
                    vval = vval / (N)
                    newval = self.Relaxation * vval + (1 - self.Relaxation) * val
                    array.SetTuple1(i,newval)
        elif self.Connexity == 2:
            for n in range (self.Iterations):
                for i in range (surfEdges.GetNumberOfPoints()):
                    cells = vtk.vtkIdList()
                    surfEdges.GetPointCells(i,cells)
                    pointlist = vtk.vtkIdList()
                    vval = 0
                    ddd = 0
                    d = 0
                    N = 0
                    for j in range (cells.GetNumberOfIds()):
                        points = vtk.vtkIdList()
                        surfEdges.GetCellPoints(cells.GetId(j),points)
                        for k in range (points.GetNumberOfIds()):
                            if points.GetId(k) != i:
                                pointlist.InsertUniqueId(points.GetId(k))
                                cells2 = vtk.vtkIdList()
                                surfEdges.GetPointCells(i,cells2)
                                for p in range (cells2.GetNumberOfIds()):
                                    points2 = vtk.vtkIdList()
                                    surfEdges.GetCellPoints(cells2.GetId(j),points2)
                                    for q in range (points2.GetNumberOfIds()):
                                        if points2.GetId(k) != i:
                                            pointlist.InsertUniqueId(points2.GetId(k))
                    N = pointlist.GetNumberOfIds()
                    for j in range (pointlist.GetNumberOfIds()):
                        d = math.sqrt(vtk.vtkMath.Distance2BetweenPoints(surface.GetPoint(i),surface.GetPoint(pointlist.GetId(j))))
                        dd = 1/d
                        val = array.GetComponent(pointlist.GetId(j),0)
                        vval = vval + val
                        ddd = ddd + dd
                    val = array.GetComponent(i,0)
                    vval = vval / (N)
                    newval = self.Relaxation * vval + (1 - self.Relaxation) * val
                    array.SetTuple1(i,newval)
        else:
            self.PrintError ('Error: wrong connexity')

        self.Surface = surface


if __name__ == '__main__':
    main = pypes.pypeMain()
    main.Arguments = sys.argv
    main.Execute()



## Program: VMTK
## Language:  Python
## Date:      February 2, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkcenterlinemodeller as centerlinemodeller


def test_default_params(aorta_centerline, compare_images):
    name = __name__ + '_test_default_params.vti'
    modeller = centerlinemodeller.vmtkCenterlineModeller()
    modeller.Centerlines = aorta_centerline
    modeller.RadiusArrayName = 'MaximumInscribedSphereRadius'
    modeller.Execute()

    assert compare_images(modeller.Image, name) == True


def test_128x_dimensions(aorta_centerline, compare_images):
    name = __name__ + '_test_48x_dimensions.vti'
    modeller = centerlinemodeller.vmtkCenterlineModeller()
    modeller.Centerlines = aorta_centerline
    modeller.RadiusArrayName = 'MaximumInscribedSphereRadius'
    modeller.SampleDimensions = [48, 48, 48]
    modeller.Execute()

    assert compare_images(modeller.Image, name) == True


def test_negate_function(aorta_centerline, compare_images):
    name = __name__ + '_test_negate_function.vti'
    modeller = centerlinemodeller.vmtkCenterlineModeller()
    modeller.Centerlines = aorta_centerline
    modeller.RadiusArrayName = 'MaximumInscribedSphereRadius'
    modeller.NegateFunction = 1
    modeller.Execute()

    assert compare_images(modeller.Image, name) == True



## Program: VMTK
## Language:  Python
## Date:      February 2, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import vmtk.vmtkcenterlineresampling as centerlineresampling


def test_default_params(aorta_centerline, compare_centerlines):
    name = __name__ + '_test_default_params.vtp'
    resampler = centerlineresampling.vmtkCenterlineResampling()
    resampler.Centerlines = aorta_centerline
    resampler.Length = 0.05
    resampler.Execute()

    assert resampler.Centerlines.GetNumberOfPoints() > 3000
    assert compare_centerlines(resampler.Centerlines, name) == True



## Program: VMTK
## Language:  Python
## Date:      January 10, 2018
## Version:   1.4

##   Copyright (c) Richard Izzo, Luca Antiga, All rights reserved.
##   See LICENSE file for details.

##      This software is distributed WITHOUT ANY WARRANTY; without even
##      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
##      PURPOSE.  See the above copyright notices for more information.

## Note: this code was contributed by
##       Richard Izzo (Github @rlizzo)
##       University at Buffalo

import pytest
import unittest
import os


@pytest.fixture(scope='function')
def vmtk_scripts():
    allscripts =  [
        'vmtk.vmtkactivetubes',
        'vmtk.vmtkbifurcationprofiles',
        'vmtk.vmtkbifurcationreferencesystems',
        'vmtk.vmtkbifurcationsections',
        'vmtk.vmtkbifurcationvectors',
        'vmtk.vmtkboundarylayer',
        'vmtk.vmtkboundaryreferencesystems',
        'vmtk.vmtkbranchclipper',
        'vmtk.vmtkbranchextractor',
        'vmtk.vmtkbranchgeometry',
        'vmtk.vmtkbranchmapping',
        'vmtk.vmtkbranchmetrics',
        'vmtk.vmtkbranchpatching',
        'vmtk.vmtkbranchsections',
        'vmtk.vmtkcenterlineattributes',
        'vmtk.vmtkcenterlinegeometry',
        'vmtk.vmtkcenterlineinterpolation',
        'vmtk.vmtkcenterlinelabeler',
        'vmtk.vmtkcenterlinemerge',
        'vmtk.vmtkcenterlinemodeller',
        'vmtk.vmtkcenterlineoffsetattributes',
        'vmtk.vmtkcenterlineresampling',
        'vmtk.vmtkcenterlines',
        'vmtk.vmtkcenterlinesnetwork',
        'vmtk.vmtkcenterlinestonumpy',
        'vmtk.vmtkcenterlinesections',
        'vmtk.vmtkcenterlinesmoothing',
        'vmtk.vmtkcenterlineviewer',
        'vmtk.vmtkdelaunayvoronoi',
        'vmtk.vmtkdistancetocenterlines',
        'vmtk.vmtkendpointextractor',
        'vmtk.vmtkflowextensions',
        'vmtk.vmtkicpregistration',
        'vmtk.vmtkimagebinarize',
        'vmtk.vmtkimagecast',
        'vmtk.vmtkimagecompose',
        'vmtk.vmtkimagecurvedmpr',
        'vmtk.vmtkimagefeaturecorrection',
        'vmtk.vmtkimagefeatures',
        'vmtk.vmtkimageinitialization',
        'vmtk.vmtkimagemipviewer',
        'vmtk.vmtkimagemorphology',
        'vmtk.vmtkimagenormalize',
        'vmtk.vmtkimageobjectenhancement',
        'vmtk.vmtkimageotsuthresholds',
        'vmtk.vmtkimagereader',
        'vmtk.vmtkimagereslice',
        'vmtk.vmtkimageseeder',
        'vmtk.vmtkimageshiftscale',
        'vmtk.vmtkimagesmoothing',
        'vmtk.vmtkimagetonumpy',
        'vmtk.vmtkimageviewer',
        'vmtk.vmtkimagevesselenhancement',
        'vmtk.vmtkimagevoipainter',
        'vmtk.vmtkimagevoiselector',
        'vmtk.vmtkimagewriter',
        'vmtk.vmtklevelsetsegmentation',
        'vmtk.vmtklineartoquadratic',
        'vmtk.vmtklineresampling',
        'vmtk.vmtklocalgeometry',
        'vmtk.vmtkmarchingcubes',
        'vmtk.vmtkmesharrayoperation',
        'vmtk.vmtkmeshboundaryinspector',
        'vmtk.vmtkmeshbranchclipper',
        'vmtk.vmtkmeshclipper',
        'vmtk.vmtkmeshconnectivity',
        'vmtk.vmtkmeshcutter',
        'vmtk.vmtkmeshdatareader',
        'vmtk.vmtkmeshextractpointdata',
        'vmtk.vmtkmeshlambda2',
        'vmtk.vmtkmeshlinearize',
        'vmtk.vmtkmeshgenerator',
        'vmtk.vmtkmeshmergetimesteps',
        'vmtk.vmtkmeshpolyballevaluation',
        'vmtk.vmtkmeshprojection',
        'vmtk.vmtkmeshreader',
        'vmtk.vmtkmeshscaling',
        'vmtk.vmtkmeshtetrahedralize',
        'vmtk.vmtkmeshtosurface',
        'vmtk.vmtkmeshtransform',
        'vmtk.vmtkmeshtransformtoras',
        'vmtk.vmtkmeshvectorfromcomponents',
        'vmtk.vmtkmeshviewer',
        'vmtk.vmtkmeshvolume',
        'vmtk.vmtkmeshvorticityhelicity',
        'vmtk.vmtkmeshwallshearrate',
        'vmtk.vmtkmeshwriter',
        'vmtk.vmtknetworkeditor',
        'vmtk.vmtknetworkextraction',
        'vmtk.vmtknetworkwriter',
        'vmtk.vmtknumpyreader',
        'vmtk.vmtknumpytocenterlines',
        'vmtk.vmtknumpytoimage',
        'vmtk.vmtknumpytosurface',
        'vmtk.vmtknumpywriter',
        'vmtk.vmtkparticletracer',
        'vmtk.vmtkpathlineanimator',
        'vmtk.vmtkpointsplitextractor',
        'vmtk.vmtkpointtransform',
        'vmtk.vmtkpolyballmodeller',
        'vmtk.vmtkpotentialfit',
        'vmtk.vmtkpythonscript',
        'vmtk.vmtkrenderer',
        'vmtk.vmtkrendertoimage',
        'vmtk.vmtkrbfinterpolation',
        'vmtk.vmtksurfaceappend',
        'vmtk.vmtksurfacearraysmoothing',
        'vmtk.vmtksurfacearrayoperation',
        'vmtk.vmtksurfacebooleanoperation',
        'vmtk.vmtksurfacecapper',
        'vmtk.vmtksurfacecelldatatopointdata',
        'vmtk.vmtksurfacecenterlineprojection',
        'vmtk.vmtksurfaceclipper',
        'vmtk.vmtksurfacecliploop',
        'vmtk.vmtksurfaceconnectivity',
        'vmtk.vmtksurfacecurvature',
        'vmtk.vmtksurfacedecimation',
        'vmtk.vmtksurfacedistance',
        'vmtk.vmtksurfaceendclipper',
        'vmtk.vmtksurfacekiteremoval',
        'vmtk.vmtksurfaceloopextraction',
        'vmtk.vmtksurfacemassproperties',
        'vmtk.vmtksurfacemodeller',
        'vmtk.vmtksurfacenormals',
        'vmtk.vmtksurfacepointdatatocelldata',
        'vmtk.vmtksurfacepolyballevaluation',
        'vmtk.vmtksurfaceprojection',
        'vmtk.vmtksurfacereader',
        'vmtk.vmtksurfacereferencesystemtransform',
        'vmtk.vmtksurfaceregiondrawing',
        'vmtk.vmtksurfaceremeshing',
        'vmtk.vmtksurfacescaling',
        'vmtk.vmtksurfacesmoothing',
        'vmtk.vmtksurfacesubdivision',
        'vmtk.vmtksurfacetonumpy',
        'vmtk.vmtksurfacetransform',
        'vmtk.vmtksurfacetransforminteractive',
        'vmtk.vmtksurfacetransformtoras',
        'vmtk.vmtksurfacetriangle',
        'vmtk.vmtksurfacetomesh',
        'vmtk.vmtksurfaceviewer',
        'vmtk.vmtksurfacewriter',
        'vmtk.vmtksurfmesh',
        'vmtk.vmtktetgen',
        'vmtk.vmtktetringenerator',
        'vmtk.vmtkboundarylayer2',
        'vmtk.vmtkcenterlinestonumpy',
        'vmtk.vmtkdijkstradistancetopoints',
        'vmtk.vmtkdistancetospheres',
        'vmtk.vmtkentityrenumber',
        'vmtk.vmtkgeodesicsurfaceresolution',
        'vmtk.vmtkimagetonumpy',
        'vmtk.vmtkmeshaddexternallayer',
        'vmtk.vmtkmeshclipcenterlines',
        'vmtk.vmtkmeshmerge',
        'vmtk.vmtkmeshtetrahedralize2',
        'vmtk.vmtkmeshviewer2',
        'vmtk.vmtkmeshwriter2',
        'vmtk.vmtknumpyreader',
        'vmtk.vmtknumpytocenterlines',
        'vmtk.vmtknumpytoimage',
        'vmtk.vmtknumpytosurface',
        'vmtk.vmtknumpywriter',
        'vmtk.vmtksurfaceextractinnercylinder',
        'vmtk.vmtksurfaceresolution',
        'vmtk.vmtksurfacetonumpy',
        'vmtk.vmtksurfacewriter2',
        'vmtk.vmtkthreshold' ]
    return allscripts


def test_import_vmtkScripts(vmtk_scripts):
    import importlib
    print(vmtk_scripts)
    for name in vmtk_scripts:
        assert importlib.import_module(name)
